/********************************************************************************
*                                                                               *
*                        G L  C o n t e x t E x  C l a s s                      *
*                                                                               *
*********************************************************************************
* Copyright (C) 2003 by ABAQUS, Inc.   All Rights Reserved.                     *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Library General Public                   *
* License as published by the Free Software Foundation; either                  *
* version 2 of the License, or (at your option) any later version.              *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Library General Public License for more details.                              *
*                                                                               *
* You should have received a copy of the GNU Library General Public             *
* License along with this library; if not, write to the Free                    *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
********************************************************************************/
#ifndef FXGLCONTEXTEX_H
#define FXGLCONTEXTEX_H
// begin local includes
#include <fgl_FGLAPI.h>
#include <FXGLContext.h>

class xmd_OGLCtx;
class FXGLImage;


/**
* A GL context is an object representing the OpenGL state information.
* Multiple GL context may share display lists to conserve memory.
* When drawing multiple windows, it may be advantageous to share not only
* display lists, but also GL contexts.  Since the GL context is created
* for a certain frame-buffer configuration, sharing of GL contexts is
* only possible if the windows sharing the GL context all have the same
* GL visual.
* However, display lists may be shared between different GL contexts.
*/
class FGLAPI FXGLContextEx : public FXGLContext {
  FXDECLARE(FXGLContextEx)

  friend class xmd_OGLCtx;

protected:
  void		 *overlayCtx; // GL overlay context
  FXbool	  overlayShares; // Overlay context and normal context share display lists.
#ifdef WIN32
  FXID		 imageDC;
#endif
protected:
  FXGLContextEx();
private:
  FXGLContextEx(const FXGLContextEx&);
  FXGLContextEx &operator=(const FXGLContextEx&);
public:

  /**
  * Construct an OpenGL context with its own private display list.
  */
  FXGLContextEx(FXApp* a,FXGLVisual *vis,FXbool wantDirect=TRUE); // CAE: added wantDirect

  /**
  * Construct an OpenGL context sharing display lists with an existing GL context.
  */
  FXGLContextEx(FXApp* a,FXGLVisual *vis,FXGLContext *shared,FXbool wantDirect=TRUE); // CAE: added wantDirect

  /// Create context 
  virtual void create();
  void create(FXID wnd);
  
  /// Detach the server-side resources for this window 
  virtual void detach();
  
  /// Destroy the server-side resources for this window 
  virtual void destroy();

  /// Return OpenGL context from FXGLContext parent.  
  void *getContext() { return ctx; }

  /// Make OpenGL overlay context current prior to performing OpenGL commands
  FXbool beginOverlay(FXDrawable *drawable);

  /// Clear OpenGL overlay planes
  void clearOverlay();
  
  /// Make OpenGL overlay context non current 
  FXbool endOverlay();

  /// Return TRUE if overlay context shares display lists with normal context
  FXbool overlaySharesDL() { return overlayShares; }
  
  /// Make OpenGL image context current prior to performing OpenGL commands
  FXbool beginImage(FXGLImage *pImage);

  /// Make OpenGL image context non current 
  FXbool endImage(FXGLImage *pImage);

  /// Destructor
  virtual ~FXGLContextEx();
  };


#endif

