// begin local includes
/********************************************************************************
*                                                                               *
*                        O p e n G L   D C   O b j e c t                        *
*                                                                               *
*********************************************************************************
* Copyright (C) 2003 by ABAQUS, Inc.   All Rights Reserved.                     *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Library General Public                   *
* License as published by the Free Software Foundation; either                  *
* version 2 of the License, or (at your option) any later version.              *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Library General Public License for more details.                              *
*                                                                               *
* You should have received a copy of the GNU Library General Public             *
* License along with this library; if not, write to the Free                    *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
********************************************************************************/
#define FGLDLL_EXPORTS
#include <xincs.h>
#include <fxver.h>
#include <fxdefs.h>
#include <FXStream.h>
#include <FXString.h>
#include <FXObject.h>
#include <FXDict.h>
#include <FXRegistry.h>
#include <FXAccelTable.h>
#include <FXApp.h>
#include <FXId.h>
#include <FXVisual.h>
#include <FXOVVisual.h>
#include <FXGLVisual.h>
#include <FXGLVisualEx.h>
#include <FXDrawable.h>
#include <FXWindow.h>
#include <FXCursor.h>
#include <FXCanvas.h>
#include <FXGLContext.h>
#include <FXGLContextEx.h>
#include <FXGLCanvas.h>
#include <FXGLDC.h>


#define DISPLAY(app) ((Display*)((app)->getDisplay()))

/*******************************************************************************/


// Object implementation
FXIMPLEMENT(FXGLDC,FXGLCanvas,NULL,0)


// For serialization
FXGLDC::FXGLDC(){
  flags|=FLAG_ENABLED|FLAG_SHOWN;
  sgnext=this;
  sgprev=this;
  context=0;
  ownContext=FALSE;
#ifndef WIN32
  overlayWindow=0;
#endif
  }


// Make a canvas
FXGLDC::FXGLDC(FXComposite* p,FXGLVisual *vis,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h):
  FXGLCanvas(p,vis,tgt,sel,opts,x,y,w,h){
  flags|=FLAG_ENABLED|FLAG_SHOWN;
  sgnext=this;
  sgprev=this;
  context=0;
  ownContext=FALSE;
#ifndef WIN32
  overlayWindow=0;
#endif
  }


// Make a canvas sharing a context
FXGLDC::FXGLDC(FXComposite* p,FXGLContextEx *inctx,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h):
  FXGLCanvas(p,inctx->getVisual(),tgt,sel,opts,x,y,w,h){
  flags|=FLAG_ENABLED|FLAG_SHOWN;
  sgnext=this;
  sgprev=this;
  context=inctx;
  ownContext=FALSE;
#ifndef WIN32
  overlayWindow=0;
#endif
  }


// Make a canvas sharing display lists
FXGLDC::FXGLDC(FXComposite* p,FXGLVisual *vis,FXGLDC* sharegroup,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h):
  FXGLCanvas(p,vis,tgt,sel,opts,x,y,w,h){
  flags|=FLAG_ENABLED|FLAG_SHOWN;
  if(sharegroup){
    sgnext=sharegroup;
    sgprev=sharegroup->sgprev;
    sharegroup->sgprev=this;
    sgprev->sgnext=this;
    }
  else{
    sgnext=this;
    sgprev=this;
    }
  context=0;
  ownContext=FALSE;
#ifndef WIN32
  overlayWindow=0;
#endif
  }



#ifdef WIN32
const char* FXGLDC::GetClass() const { return "FXGLCanvas"; }
#endif


// Return TRUE if it is sharing display lists
FXbool FXGLDC::isShared() const{ 
		      // FIXME:  Is this desired?
  return sgnext!=this /*|| (context && context->sgnext!=context)*/;
  }


// Create X window (GL CANVAS)
void FXGLDC::create(){
  FXGLDC *gldc=this;
  void *sharedctx=NULL;
#if 0
  FXGLCanvas::create();
#else
  
  FXCanvas::create();
#if defined(HAVE_OPENGL) && defined(WIN32)
  HDC hdc=::GetDC((HWND)xid);
  if(!SetPixelFormat(hdc,visual->pixelformat,(PIXELFORMATDESCRIPTOR*)visual->info)){ 
    fxerror("%s::create(): SetPixelFormat() failed.\n",getClassName()); 
    }
  ::ReleaseDC((HWND)xid,hdc);
#endif
  
#endif
  if(!context){
    if(!visual->info){ fxerror("%s::create(): visual unsuitable for OpenGL.\n",getClassName()); }
    if(sgnext!=this){
      
      // Find another member of the group which is already created, and get its context
      gldc=sgnext;
      while(gldc!=this){
        sharedctx=gldc->context;
        if(sharedctx) break;
        gldc=gldc->sgnext;
        }
      }
    // Make context
    context = new FXGLContextEx(getApp(),(FXGLVisual *)visual,gldc!=this?gldc->context:NULL);
    if(!context){ fxerror("%s::create(): Memory allocation failed.\n",getClassName()); }
    else ownContext=TRUE;
    }
  context->create(xid);
#ifndef WIN32
  if( 0==overlayWindow && (VISUAL_OVERLAY&visual->flags) ){
    // Create window for overlay graphics
    FXOVVisual *visOV = ((FXGLVisualEx*)(context->getVisual()))->getOverlayVisual();
    if( visOV && 0<visOV->getActualOverlaySize() ){
      // Create Overlay window
      XSetWindowAttributes wattrs;
      wattrs.border_pixel     = visOV->clearPixel;
      wattrs.background_pixel = wattrs.border_pixel;
      wattrs.colormap         = visOV->colormap;
      wattrs.save_under       = False;

      FXTRACE((100,"FXGLDC::create : Make %dx%d overlay window\n",width,height));

      overlayWindow = (FXID) XCreateWindow( DISPLAY(getApp()), (Window)xid,
		0, 0, FXMAX(width,1),FXMAX(height,1),
		0, ((XVisualInfo *)(visOV->info))->depth,
		InputOutput, ((XVisualInfo *)(visOV->info))->visual,
		CWBorderPixel | CWBackPixel | CWSaveUnder | CWColormap,
		&wattrs );

      // TBD: Should overlay be abandoned on the context?

      if( overlayWindow ){
#if 0
	// Check to see if there is already a property
	Window windows[2],*windowsReturn,*windowList;
	int countReturn,i;
	FXWindow *shellWin = getShell();
	if( NULL == shellWin )
	    shellWin = getParent();
	Status status=XGetWMColormapWindows(DISPLAY(getApp()),shellWin->id(),&windowsReturn,&countReturn);

	// If no property, just create one
	if(!status){
	  windows[0]=(Window)overlayWindow;
	  windows[1]=getShell()->id();
	  XSetWMColormapWindows(DISPLAY(getApp()),shellWin->id(),windows,2);
	  }
	else{
	  // There was a property, add myself to the end
	  windowList=(Window*)malloc((sizeof(Window))*(countReturn+1));
	  for(i=0; i<countReturn; i++) windowList[i]=windowsReturn[i];
	  windowList[i]=(Window)overlayWindow;
	  XSetWMColormapWindows(DISPLAY(getApp()),shellWin->id(),windowList,countReturn+1);
	  XFree((char*)windowsReturn);
	  free(windowList);
	  }
#endif // 0

	XMapRaised(DISPLAY(getApp()),(Window)overlayWindow);
	}
      }
    }
#endif
  }

#ifndef WIN32
void FXGLDC::position(FXint x, FXint y, FXint w, FXint h){
  FXbool resizeOverlay = (NULL != overlayWindow && 0<w && 0<h && w!=width && h!=height);
  FXGLCanvas::position(x,y,w,h);    
  if(resizeOverlay){
    XResizeWindow(DISPLAY(getApp()), (Window)overlayWindow, width, height);
    }
  }

void FXGLDC::resize(FXint w, FXint h){
  FXbool resizeOverlay = (NULL != overlayWindow && 0<w && 0<h && w!=width && h!=height);
  FXGLCanvas::resize(w,h);    
  if(resizeOverlay){
    XResizeWindow(DISPLAY(getApp()), (Window)overlayWindow, width, height);
    }
  }
#endif


// Detach the GL Canvas
void FXGLDC::detach(){
  if(context){
    // Will this leak memory?
    context->detach();
    }
  FXGLCanvas::detach();
  }


// Destroy the GL Canvas
void FXGLDC::destroy(){
  if(context && ownContext)
      context->destroy();
  FXGLCanvas::destroy();
  }


//  Make the rendering context of GL Canvas current
FXbool FXGLDC::makeCurrent(){
  if(context){
    return context->begin(this);
    }
  return FALSE;
  }
  
  
//  Make the rendering context of GL Canvas current
FXbool FXGLDC::makeNonCurrent(){
  if(context){
    return context->end();
    }
  return FALSE;
  }


// Used by GL to swap the buffers in double buffer mode, or flush a single buffer
void FXGLDC::swapBuffers(){
  if(context){
    context->swapBuffers();
    }
  }


// Save object to stream
void FXGLDC::save(FXStream& store) const {
  FXGLCanvas::save(store);
  store << sgnext;
  store << sgprev;
  }

      
// Load object from stream
void FXGLDC::load(FXStream& store){
  FXGLCanvas::load(store);
  store >> sgnext;
  store >> sgprev;
  }


// Close and release any resources
FXGLDC::~FXGLDC(){
  sgnext->sgprev=sgprev;
  sgprev->sgnext=sgnext;
  sgnext=(FXGLDC*)-1;
  sgprev=(FXGLDC*)-1;

  // Only delete the context if we created it.
  if(ownContext) delete context;
#ifndef WIN32
  if( None != overlayWindow ) {

    // Get list of colormapped windows and remove overlay window
    Window *windowsReturn;
    int countReturn, i, j;
    Status status=XGetWMColormapWindows(DISPLAY(getApp()),getShell()->id(),&windowsReturn,&countReturn);
    if(status){
      for(i=0,j=0; i<countReturn; i++) {
        if( windowsReturn[i] != (Window)overlayWindow )
	  windowsReturn[j++] = windowsReturn[i];
        }
      XSetWMColormapWindows(DISPLAY(getApp()),getShell()->id(),windowsReturn,j);
      XFree((char*)windowsReturn);
      }

    XDestroyWindow(DISPLAY(getApp()), (Window)overlayWindow);
    overlayWindow = 0;
    }
#endif
  }
