/********************************************************************************
*                                                                               *
*                            G L   D C   O b j e c t                            *
*                                                                               *
*********************************************************************************
* Copyright (C) 2003 by ABAQUS, Inc.   All Rights Reserved.                     *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Library General Public                   *
* License as published by the Free Software Foundation; either                  *
* version 2 of the License, or (at your option) any later version.              *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Library General Public License for more details.                              *
*                                                                               *
* You should have received a copy of the GNU Library General Public             *
* License along with this library; if not, write to the Free                    *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
********************************************************************************/
#ifndef FXGLDC_H
#define FXGLDC_H
// begin local includes
#include <fgl_FGLAPI.h>
#include <FXVisual.h>
#include <FXGLCanvas.h>

class FXGLContextEx;

/// GLCanvas, an area drawn by another object
class FGLAPI FXGLDC : public FXGLCanvas {
  FXDECLARE(FXGLDC)
private:
  FXGLDC	*sgnext;  // Share group next in share list
  FXGLDC	*sgprev;  // Share group previous in share list
protected:
#ifndef WIN32
  FXID		overlayWindow;
#endif
  FXGLContextEx	*context; // GL Context
  FXbool	ownContext;
  FXGLDC();
private:
  FXGLDC(const FXGLDC&);
  FXGLDC &operator=(const FXGLDC&);
#ifdef WIN32
  virtual const char* GetClass() const;
#endif
public:

  /**
  * Construct an OpenGL-capable canvas, with its own private display list.
  */
  FXGLDC(FXComposite* p,FXGLVisual *vis,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

  /**
  * Construct an OpenGL-capable canvas, with a shared OpenGL context.
  */
  FXGLDC(FXComposite* p,FXGLContextEx *sctx,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

  /**
  * Construct an OpenGL-capable canvas, sharing display
  * list with another GL canvas.  This canvas becomes a member
  * of a display list share group.  All members of the display
  * list share group have to have the same visual.
  */
  FXGLDC(FXComposite* p,FXGLVisual *vis,FXGLDC* sharegroup,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

  /// Return TRUE if it is sharing display lists
  FXbool isShared() const;
  
  /// Create all of the server-side resources for this window 
  virtual void create();
  
  /// Detach the server-side resources for this window 
  virtual void detach();
  
  /// Destroy the server-side resources for this window 
  virtual void destroy();
  
  /// Make OpenGL context current prior to performing OpenGL commands
  virtual FXbool makeCurrent();
  
  /// Make OpenGL context non current
  virtual FXbool makeNonCurrent();
  
  /// Swap front and back buffer
  virtual void swapBuffers();

#ifndef WIN32
  /// Get overlay window
  FXID getOverlayWindow()   { return overlayWindow; }

  virtual void position(FXint x, FXint y, FXint w, FXint h);

  virtual void resize(FXint w, FXint h);
#else

  /// Get overlay window
  FXID getOverlayWindow()  { return (VISUAL_OVERLAY&visual->flags) ? xid : 0; }
#endif

  /// Save object to stream
  virtual void save(FXStream& store) const;

  /// Load object from stream
  virtual void load(FXStream& store);

  /// Destructor
  virtual ~FXGLDC();
  };


#endif

