#ifndef WIN32
#include <osmesa.h>
#endif
// begin local includes
/********************************************************************************
*                                                                               *
*                        G L   I m a g e    O b j e c t                         *
*                                                                               *
*********************************************************************************
* Copyright (C) 2003 by ABAQUS, Inc.   All Rights Reserved.                     *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Library General Public                   *
* License as published by the Free Software Foundation; either                  *
* version 2 of the License, or (at your option) any later version.              *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Library General Public License for more details.                              *
*                                                                               *
* You should have received a copy of the GNU Library General Public             *
* License along with this library; if not, write to the Free                    *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
********************************************************************************/
#define FGLDLL_EXPORTS
#include <xincs.h>
#include <fxver.h>
#include <fxdefs.h>
#include <FXStream.h>
#include <FXString.h>
#include <FXSize.h>
#include <FXPoint.h>
#include <FXRectangle.h>
#include <FXObject.h>
#include <FXDict.h>
#include <FXRegistry.h>
#include <FXApp.h>
#include <FXId.h>
#include <FXVisual.h>
#include <FXDrawable.h>
#include <FXImage.h>
#include <FXGLImage.h>
#include <FXDC.h>
#include <FXDCWindow.h>


/*
  Notes:
  - FXGLImage::create() renders rgb[a] data into X/GDI resident, device
    dependent pixmap.
  - Need to be able to re-render subpart of image.
  - We should implement shared pixmaps.
  - If IMAGE_KEEP, repeated rendering is usually desired; should we
    hang on to XImage, and the shared memory segment in that case?
    How about shared pixmaps...
  - Slight change in interpretation of IMAGE_OWNED flag:- if passed, the
    FXGLImage will own client-side pixel buffer, otherwise it will not; if
    no pixel-buffer has been passed and IMAGE_OWNED *is* passed, a pixel
    buffer will be allocated [and cleared to zero].
    No pixel buffer will be allocated if neither IMAGE_OWNED nor pixels
    are passed.
  - When using shared image/pixmaps, if IMAGE_KEEP is set, hang on to pixel buffer.
  - Need resize, rotate member functions.
  - We need to speed up 16/15 bpp true color.
  - We need dither tables with 3bit and 2bit rounding for 5,6,5/5,5,5 modes
  - We need dither tables with 5bit, 6bit rounding for 3,3,2 mode.
  - We need to split true color from direct color, because direct color
    has random mapping, true has not.
  - Just because I always forget:
  
      StaticGray   0
      GrayScale    1
      StaticColor  2
      PseudoColor  3
      TrueColor    4
      DirectColor  5
*/


#define DISPLAY(app) ((Display*)((app)->getDisplay()))

  
/*******************************************************************************/

// Object implementation
FXIMPLEMENT(FXGLImage,FXImage,NULL,0)


// For deserialization
FXGLImage::FXGLImage(){
  baseXID=0;
  }


// Initialize
FXGLImage::FXGLImage(FXApp* a,const void *pix,FXuint opts,FXint w,FXint h):FXImage(a,pix,opts,w,h){
  FXTRACE((100,"FXGLImage::FXGLImage %08x\n",this));
  baseXID=0;
  }


// Create image
void FXGLImage::create(){
  if(!xid){
    FXTRACE((100,"FXGLImage::create %08x\n",this));

#ifndef WIN32
    if(VISUAL_OSMESA&visual->getFlags()){
      // Initialize visual
      visual->create();

      // Allocate memory for pixels
      if( ! data ){
	FXCALLOC(&data,FXuchar,width*height*channels);
	options |= IMAGE_OWNED;
	}
      xid = 1;
      }
#ifdef GLX_PBUFFER
    else if(VISUAL_PBUFFER&visual->getFlags()){
      // Initialize visual
      visual->create();
      if(visual->info){
        int anPbufferAttribs[] = {
		GLX_PBUFFER_WIDTH,      500,
		GLX_PBUFFER_HEIGHT,     500,
		GLX_PRESERVED_CONTENTS,   1,
		GLX_LARGEST_PBUFFER,      1,
		0
	    };
        anPbufferAttribs[1] = width;
        anPbufferAttribs[3] = height;
        xid=glXCreatePbuffer(DISPLAY(getApp()),*((GLXFBConfig*)(visual->info)),anPbufferAttribs);
        }
      if(!xid){ fxwarning("%s::create: unable to create %dx%d PBuffer image.\n",getClassName(),width,height); return; }
#ifndef NDEBUG
      else{
	FXTRACE((150,"FXGLImage::create %dx%d PBuffer 0x%08x\n",width,height, xid));
	unsigned int nPbWidth,nPbHeight,nPbPreserve,nPbLargest;
	glXQueryDrawable(DISPLAY(getApp()),(GLXPbuffer)xid, GLX_WIDTH, &nPbWidth);
	glXQueryDrawable(DISPLAY(getApp()),(GLXPbuffer)xid, GLX_HEIGHT, &nPbHeight);
	glXQueryDrawable(DISPLAY(getApp()),(GLXPbuffer)xid, GLX_PRESERVED_CONTENTS, &nPbPreserve);
	glXQueryDrawable(DISPLAY(getApp()),(GLXPbuffer)xid, GLX_LARGEST_PBUFFER, &nPbLargest);
	FXTRACE((150,"\tSize:     %ux%u\n\tPreserve: %s\n\tLargest:  %s\n",
	    nPbWidth, nPbHeight, nPbPreserve?"Yes":"No",nPbLargest?"Yes":"No"));
        }
#endif
      }
#endif
    else {
      FXImage::create();

      if(xid && (VISUAL_IMAGE & visual->getFlags())){
        baseXID=xid;
        xid=glXCreateGLXPixmap(DISPLAY(getApp()),(XVisualInfo *)visual->info,xid);
        if(!xid){
	  xid = baseXID;
	  baseXID = 0;
	  fxwarning("%s::create: unable to create GLXPixmap image.\n",getClassName());
	  return;
	  }
        }
      }

#else

    if(VISUAL_IMAGE&visual->getFlags()){
      // Initialize visual
      visual->create();

      // Create a bitmap compatible with current display
      BITMAPINFOHEADER bih  = { sizeof(BITMAPINFOHEADER),
				width, height, 1,
				32, BI_RGB, 0,
				0, 0, 0, 0 };
      BITMAPINFO bi;
      void *biData=NULL;
      bi.bmiHeader = bih;
      HDC hdc=::CreateCompatibleDC(NULL);
      xid=CreateDIBSection(hdc,&bi, DIB_RGB_COLORS, &biData, NULL, 0);
      ::DeleteDC(hdc);
      if(!xid){ fxerror("%s::create: unable to create image.\n",getClassName()); }
      }
    else
      FXImage::create();
#endif
    }
  }


// Detach image
void FXGLImage::detach(){
  if(baseXID){
    FXTRACE((100,"%s::detach %08x\n",getClassName(),this));
    xid=baseXID;
    baseXID=0;
    }
  FXImage::detach();
  }


// Destroy image
void FXGLImage::destroy(){
#ifndef WIN32
  if(baseXID){
    FXTRACE((100,"FXGLImage::destroy %08x\n",this));
    glXDestroyGLXPixmap(DISPLAY(getApp()),xid);
    xid=baseXID;
    baseXID=0;
    }
  if(VISUAL_OSMESA&visual->getFlags()){
    FXTRACE((100,"FXGLImage::destroy %08x\n",this));
    if( data && (options&IMAGE_OWNED) ){
      FXFREE(&data);
      data = NULL;
      }
    xid = 0;
    }
#ifdef GLX_PBUFFER
  else if(VISUAL_PBUFFER&visual->getFlags()){
    if(xid){
      glXDestroyPbuffer( DISPLAY(getApp()), (GLXPbuffer)xid );
      xid = 0;
      }
    }
#endif
  else
#endif
  FXImage::destroy();
  }


// Restore client-side pixel buffer from image
void FXGLImage::restore(){
    FXTRACE((100,"FXGLImage::restore image %p\n",this));
#ifndef WIN32
  if(VISUAL_OSMESA&visual->getFlags()){
    // OSMesa renders directly into the 'data' so it is already 'restore'd.
    // Just verify everything was set up correctly.
    FXASSERT(id() && width>=1 && height>=1 && OSMesaGetCurrentContext());
    glFinish();
    return;
    }
#endif

  if( id() && width>=1 && height>=1 &&
#ifndef WIN32
#ifndef ABQ_LINUX64  // glXGetCurrentDrawable() is broken on RedHat
      glXGetCurrentDrawable()==id() &&
#endif
      (baseXID||(VISUAL_PBUFFER&visual->getFlags())) ) {
#else
      // On NT have to assume we are current if anything is
      wglGetCurrentDC()){
#endif
    // NOTE:  If we do not get here and we wanted ALPHA, too bad.

    // Make array for data if needed
    if(!data || !(options&IMAGE_OWNED)){
      int size=width*height*channels;
      FXMALLOC(&data,FXuchar,size);
      options|=IMAGE_OWNED;
      }

    GLint nSwapBytes, nLSBFirst, nRowLength;
    GLint nSkipRows, nSkipPixels, nAlignment;

    glGetIntegerv(GL_PACK_SWAP_BYTES,  &nSwapBytes);
    glGetIntegerv(GL_PACK_LSB_FIRST,   &nLSBFirst);
    glGetIntegerv(GL_PACK_ROW_LENGTH,  &nRowLength);
    glGetIntegerv(GL_PACK_SKIP_ROWS,   &nSkipRows);
    glGetIntegerv(GL_PACK_SKIP_PIXELS, &nSkipPixels);
    glGetIntegerv(GL_PACK_ALIGNMENT,   &nAlignment);

    if(nSwapBytes !=GL_FALSE) glPixelStorei(GL_PACK_SWAP_BYTES,  GL_FALSE);
    if(nLSBFirst  !=GL_FALSE) glPixelStorei(GL_PACK_LSB_FIRST,   GL_FALSE);
    if(nRowLength !=GL_FALSE) glPixelStorei(GL_PACK_ROW_LENGTH,  GL_FALSE);
    if(nSkipRows  !=GL_FALSE) glPixelStorei(GL_PACK_SKIP_ROWS,   GL_FALSE);
    if(nSkipPixels!=GL_FALSE) glPixelStorei(GL_PACK_SKIP_PIXELS, GL_FALSE);
    if(nAlignment !=1)        glPixelStorei(GL_PACK_ALIGNMENT,   1);
    glFinish();

    glReadBuffer(GL_FRONT);  // Shouldn't be necessary
    glReadPixels(0, 0, width, height, (options&IMAGE_ALPHA)?GL_RGBA:GL_RGB,
    	GL_UNSIGNED_BYTE, (GLvoid *)data);

    if(nSwapBytes !=GL_FALSE) glPixelStorei(GL_PACK_SWAP_BYTES,  nSwapBytes);
    if(nLSBFirst  !=GL_FALSE) glPixelStorei(GL_PACK_LSB_FIRST,   nLSBFirst);
    if(nRowLength !=GL_FALSE) glPixelStorei(GL_PACK_ROW_LENGTH,  nRowLength);
    if(nSkipRows  !=GL_FALSE) glPixelStorei(GL_PACK_SKIP_ROWS,   nSkipRows);
    if(nSkipPixels!=GL_FALSE) glPixelStorei(GL_PACK_SKIP_PIXELS, nSkipPixels);
    if(nAlignment !=1)        glPixelStorei(GL_PACK_ALIGNMENT,   nAlignment);
    }
  else
    FXImage::restore();
  }


// Resize pixmap to the specified width and height; the data
// array is resized also, but its contents will be undefined.
void FXGLImage::resize(FXint w,FXint h){
  FXTRACE((100,"FXGLImage::resize(%d,%d) %p\n",w,h,this));
  if(w<1) w=1;
  if(h<1) h=1;
  if(width!=w || height!=h){
    if(id()){
      destroy();
      }
    if(data){
      if(!(options&IMAGE_OWNED)){       // Need to own array
        FXMALLOC(&data,FXuchar,w*h*channels);
        options|=IMAGE_OWNED;
        }
      else if(w*h!=width*height){
        FXRESIZE(&data,FXuchar,w*h*channels);
        }
      }
    width=w;
    height=h;
    if(!id()){
        create();
      }
    }
  }


// Clean up
FXGLImage::~FXGLImage(){
  FXTRACE((100,"FXGLImage::~FXGLImage %08x\n",this));
  destroy();
  }
