#ifndef WIN32
#include <osmesa.h>
#endif
// begin local includes
/********************************************************************************
*                                                                               *
*                            V i s u a l E x  C l a s s                         *
*                                                                               *
*********************************************************************************
* Copyright (C) 2003 by ABAQUS, Inc.   All Rights Reserved.                     *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Library General Public                   *
* License as published by the Free Software Foundation; either                  *
* version 2 of the License, or (at your option) any later version.              *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Library General Public License for more details.                              *
*                                                                               *
* You should have received a copy of the GNU Library General Public             *
* License along with this library; if not, write to the Free                    *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
********************************************************************************/
#define FGLDLL_EXPORTS
#include <xincs.h>
#include <fxver.h>
#include <fxdefs.h>
#include <fxkeys.h>
#include <FXStream.h>
#include <FXString.h>
#include <FXObject.h>
#include <FXDict.h>
#include <FXRegistry.h>
#include <FXAccelTable.h>
#include <FXObjectList.h>
#include <FXApp.h>
#include <FXId.h>
#include <FXVisual.h>
#include <FXOVVisual.h>
#include <FXGLVisual.h>
#include <FXGLVisualEx.h>


/*
  Notes:
  
  - FXGLVisualEx adds the capability for 'overlay planes' to
    FXGLVisual.  Request for overlay is rated at same level as
    request for stencil.
*/


/*******************************************************************************/


// Object implementation
FXIMPLEMENT(FXGLVisualEx,FXGLVisual,NULL,0)



// Deserialization
FXGLVisualEx::FXGLVisualEx(){
  overlay=NULL;
  }


// Construct 
FXGLVisualEx::FXGLVisualEx(FXApp* a,FXuint flgs, FXint vId):FXGLVisual(a,flgs,vId){
  FXTRACE((100,"FXGLVisualEx::FXGLVisualEx %08x\n",this));
  overlaySize=flgs&VISUAL_OVERLAY?2:0;
  overlay=NULL;
#ifndef WIN32
  if(flags&VISUAL_OSMESA){
    OSMesaContext osmctx = OSMesaCreateContext(OSMESA_RGBA, NULL);
    if(osmctx) OSMesaDestroyContext(osmctx);
    else{ flags &= ~VISUAL_OSMESA;
      FXTRACE((150,"FXGLVisualEx::FXGLVisualEx: OSMesa is not available\n"));
      }
    }
#endif
  }


/*******************************************************************************/
#ifndef WIN32
// Standard dither kernel
static const FXint dither[16]={
   0*16,  8*16,  2*16, 10*16,
  12*16,  4*16, 14*16,  6*16,
   3*16, 11*16,  1*16,  9*16,
  15*16,  7*16, 13*16,  5*16,
  };

// Apply gamma correction to an intensity value in [0..max].
static FXuint gamma_adjust(FXdouble gamma,FXuint value,FXuint max){
  register FXdouble x=(FXdouble)value / (FXdouble)max;
  return (FXuint) (((FXdouble)max * pow(x,1.0/gamma))+0.5);
  }

void FXGLVisualEx::setuposmesacolor(){
  register FXuint  redshift,greenshift,blueshift;
  register FXPixel redmask,greenmask,bluemask;
  register FXPixel redmax,greenmax,bluemax;
  register FXuint i,c,mapsize,d,r,g,b;
  register FXdouble gamma;

  // Get gamma
  gamma=getApp()->reg().readRealEntry("SETTINGS","displaygamma",1.0);

  // Get map size
  mapsize=256;

  // Arrangement of pixels
  redmask=0xff000000;
  greenmask=0x00ff0000;
  bluemask=0x0000ff00;
  redshift=24;
  greenshift=16;
  blueshift=8;
  redmax=redmask>>redshift;
  greenmax=greenmask>>greenshift;
  bluemax=bluemask>>blueshift;
  numred=redmax+1;
  numgreen=greenmax+1;
  numblue=bluemax+1;
  numcolors=numred*numgreen*numblue;

  // Make dither tables
  for(d=0; d<16; d++){
    for(i=0; i<256; i++){
      c=gamma_adjust(gamma,i,255);
      r=(redmax*c+dither[d])/255;
      g=(greenmax*c+dither[d])/255;
      b=(bluemax*c+dither[d])/255;
      rpix[d][i]=r << redshift;
      gpix[d][i]=g << greenshift;
      bpix[d][i]=b << blueshift;
      }
    }

  // What did we get
  FXTRACE((150,"OSmesa color:\n"));
  FXTRACE((150,"  depth        = %d\n",depth));
  FXTRACE((150,"  gamma        = %6f\n",gamma));
  FXTRACE((150,"  map_entries  = %d\n",mapsize));
  FXTRACE((150,"  numcolors    = %d\n",numcolors));
  FXTRACE((150,"  redmax       = %3d; redmask   =%08x; redshift   = %-2d\n",redmax,redmask,redshift));
  FXTRACE((150,"  greenmax     = %3d; greenmask =%08x; greenshift = %-2d\n",greenmax,greenmask,greenshift));
  FXTRACE((150,"  bluemax      = %3d; bluemask  =%08x; blueshift  = %-2d\n",bluemax,bluemask,blueshift));

  // Set type
  type=VISUALTYPE_TRUE;
  }
#endif

// Initialize
void FXGLVisualEx::create(){
  if(!xid){
    FXTRACE((100,"FXGLVisualEx::create\n"));
    
    // Currently this class only exists to add support for
    // overlay, off-screen (image), or OSmesa.
    // If those are not requested, revert to parent class.
    if(!(flags&(VISUAL_OVERLAY|VISUAL_IMAGE
#ifndef WIN32
	|VISUAL_OSMESA
#endif
	))) {
      FXGLVisual::create();
      return;
      }

#ifdef HAVE_OPENGL
#ifndef WIN32

    if(!(flags&VISUAL_OSMESA)) {
      // For X11, selection of base visual has no effect on selection
      // of overlay visual.  If you start running into problems with
      // the base visual being on the overlay plane then you may want
      // to replace this call to the parent class and correct that.
      FXGLVisual::create();
      }
    else {
      // Turn off incompatable flags
      flags &= ~(VISUAL_PBUFFER|VISUAL_OVERLAY);

      // DEFAULTS:  redSize = greenSize = blueSize = 8;
      // DEFAULTS:  alphaSize = 0;
      // DEFAULTS:  depthSize = 16;
      // DEFAULTS:  stencilSize = 0;
      // DEFAULTS:  accumRedSize = accumGreenSize = accumBlueSize = accumAlphaSize = 0;

      // DEFAULTS:  hint = 32;
      depth = redSize + greenSize + blueSize + alphaSize;  // Color depth
      setuposmesacolor();
      xid=1;
      }

#else

    PIXELFORMATDESCRIPTOR pfd;
    HDC hdc;
    HBITMAP hbm(NULL), hOldBm;
    int gdblbuf,glstereo,glcbits,gldepth,glaccel;      
    int glred,glgreen,glblue,glalpha,glstencil;
    int glaccred,glaccgreen,glaccblue,glaccalpha;
    int gloverlay,doverlay;
    int bestvis,dmatch,bestmatch;
    int dred,dgreen,dblue,dalpha,dcbits,ddepth,dstencil;
    int daccred,daccgreen,daccblue,daccalpha;
    int nvi,i;
    
    // Get some HDC
    if(flags&VISUAL_IMAGE){
      hdc=::CreateCompatibleDC(NULL);
      hbm=::CreateCompatibleBitmap(hdc,100,100);
      hOldBm=(HBITMAP)::SelectObject(hdc,(HBITMAP)hbm);
      }
    else{
      hdc=::GetDC(GetDesktopWindow());
      }

    
    // Get number of supported pixel formats
    pfd.nSize=sizeof(PIXELFORMATDESCRIPTOR);   
    pfd.nVersion=1;
    nvi=DescribePixelFormat(hdc,1,sizeof(PIXELFORMATDESCRIPTOR),&pfd);
    if(nvi==0){ fxerror("%s::create: no OpenGL visual available.\n",getClassName()); }

    // GL Support is required.
    DWORD dwRequiredFlags = PFD_SUPPORT_OPENGL;
    // If VISUAL_IMAGE is specified, we seek bitmap formats;
    dwRequiredFlags |= (flags&VISUAL_IMAGE) ? PFD_DRAW_TO_BITMAP : PFD_DRAW_TO_WINDOW;
    // If VISUAL_NOACCEL is specified, we skip hardware accelerated formats;
    // this is useful if you want to avoid using broken graphics hardware drivers.
    if(flags&(VISUAL_NOACCEL|VISUAL_IMAGE)) dwRequiredFlags |= PFD_GENERIC_FORMAT;

    // Try to find the best
    bestvis=-1;
    bestmatch=1000000000;

    if(userVisualId && (userVisualId >= 0) && (userVisualId <= nvi)){
	int retVal = DescribePixelFormat(hdc,userVisualId,sizeof(PIXELFORMATDESCRIPTOR),&pfd);
	if(retVal){
	    if((pfd.iPixelType==PFD_TYPE_RGBA) && ((pfd.dwFlags&dwRequiredFlags) == dwRequiredFlags) && (pfd.cDepthBits > 15)){
		bestvis = userVisualId;
	     }
	    else{
		fprintf(stderr," The user requested visual id 0x%x is unsuitable and ignored.\n", userVisualId);
		userVisualId = 0;
	    }
	}
    }

    if(userVisualId && bestvis == -1) {
        fprintf(stderr," The user requested visual id 0x%x was not found and ignored.\n", userVisualId);
        userVisualId = 0;
      }

    if(!userVisualId){
	for(i=1; i<=nvi; i++) {

	  // Get info about this visual
	  DescribePixelFormat(hdc,i,sizeof(PIXELFORMATDESCRIPTOR),&pfd);

	  // GL RGBA Support is required along with specified flags.
	  if((pfd.iPixelType==PFD_TYPE_RGBA) && ((pfd.dwFlags&dwRequiredFlags) == dwRequiredFlags)){

	    // Is this a hardware-accelerated visual?
	    glaccel=(pfd.dwFlags&PFD_GENERIC_FORMAT)==0;

	    // Must have single/double buffer as requested
	    gdblbuf=(pfd.dwFlags&PFD_DOUBLEBUFFER)!=0;

	    // Stereo support as requested
	    glstereo=(pfd.dwFlags&PFD_STEREO)!=0;

	    // Get planes
	    glcbits=pfd.cColorBits;
	    glred=pfd.cRedBits;
	    glgreen=pfd.cGreenBits;
	    glblue=pfd.cBlueBits;
	    glalpha=pfd.cAlphaBits;
	    gldepth=pfd.cDepthBits;
	    glstencil=pfd.cStencilBits;
	    glaccred=pfd.cAccumRedBits;
	    glaccgreen=pfd.cAccumGreenBits;
	    glaccblue=pfd.cAccumBlueBits;
	    glaccalpha=pfd.cAccumAlphaBits;

	    // Get number of overlay colors if overlay requested
	    gloverlay=0;
	    if(flags&VISUAL_OVERLAY) {
	      LAYERPLANEDESCRIPTOR lpd;
	      if (wglDescribeLayerPlane(hdc, i, 1, sizeof(LAYERPLANEDESCRIPTOR), &lpd)){
		if (LPD_TYPE_COLORINDEX == lpd.iPixelType)
		  gloverlay = lpd.cColorBits;
		}
	      }

	    // We prefer to get a few MORE bits in RGBA than we asked for
	    dred   = glred-redSize;     if(dred<0)   dred   *= -100;
	    dgreen = glgreen-greenSize; if(dgreen<0) dgreen *= -100;
	    dblue  = glblue-blueSize;   if(dblue<0)  dblue  *= -100;
	    dalpha = glalpha-alphaSize; if(dalpha<0) dalpha *= -100;
	    dcbits = glcbits-(redSize+greenSize+blueSize+alphaSize);
					if(dcbits<0) dcbits *= -100;

	    // Prefer better Z than asked, but colors more important
	    ddepth = gldepth-depthSize; if(ddepth<0) ddepth *= -10;

	    // We care about colors and Z depth more than stencil depth
	    dstencil = glstencil-stencilSize; if(dstencil<0) dstencil *= -1;
	    
	    // Prefer better overlay than asked, but it is least important
	    doverlay = gloverlay-overlaySize; if(doverlay<0) doverlay *= -1;

	    // Accumulation buffers
	    daccred=glaccred-accumRedSize;       if(daccred<0)   daccred   *= -1;
	    daccgreen=glaccgreen-accumGreenSize; if(daccgreen<0) daccgreen *= -1;
	    daccblue=glaccblue-accumBlueSize;    if(daccblue<0)  daccblue  *= -1;
	    daccalpha=glaccalpha-accumAlphaSize; if(daccalpha<0) daccalpha *= -1;

	    // Want the best colors, of course
	    dmatch=dred+dgreen+dblue+dalpha+dcbits;

	    // Accumulation buffers
	    dmatch+=daccred+daccgreen+daccblue+daccalpha;

	    // Hardware accelerated a plus
	    if(!glaccel){
	      dmatch+=30000000;
	      }

	    // Want alpha
	    if(alphaSize>0){
	      if(glalpha<1) dmatch+=1000000;
	      else dmatch+=dalpha;
	      }
	    else{
	      if(glalpha>0) dmatch+=1000000;
	      }

	    // Wanted Z-buffer
	    if(depthSize>0){
	      if(gldepth<1) dmatch+=100000000;
	      else dmatch+=ddepth;
	      }
	    else{
	      if(gldepth>0) dmatch+=100000000;
	      }

	    // Double buffering also quite strongly preferred
	    if(flags&VISUAL_DOUBLEBUFFER){
	      if(!gdblbuf) dmatch+=10000000;
	      }
	    else{
	      if(gdblbuf) dmatch+=10000000;
	      }

	    // Stencil buffers desired
	    if(stencilSize>0){
	      if(glstencil<1) dmatch+=10000;
	      else dmatch+=dstencil;
	      }
	    else{
	      if(glstencil>0) dmatch+=10000;
	      }
	    
	    // Overlay buffers desired
	    if(overlaySize>0){
	      if(gloverlay<1) dmatch+=10000;
	      else dmatch+=doverlay;
	      }
	    else{
	      if(gloverlay>0) dmatch+=10000;
	      }

	    // Stereo not so important
	    if(flags&VISUAL_STEREO){
	      if(!glstereo) dmatch+=100000;
	      }
	    else{
	      if(glstereo) dmatch+=100000;
	      }

	    // Trace
	    FXTRACE((100,"Pixel Format (%d) match value = %d\n",i,dmatch));
	    FXTRACE((100,"  color size  = %d\n",glcbits));
	    FXTRACE((100,"  red size    = %d\n",glred));
	    FXTRACE((100,"  green size  = %d\n",glgreen));
	    FXTRACE((100,"  blue size   = %d\n",glblue));
	    FXTRACE((100,"  alpha size  = %d\n",glalpha));
	    FXTRACE((100,"  depth size  = %d\n",gldepth));
	    FXTRACE((100,"  double buf  = %d\n",gdblbuf));
	    FXTRACE((100,"  stencil     = %d\n",glstencil));
	    FXTRACE((100,"  overlay     = %d\n",gloverlay));
	    FXTRACE((100,"  acc red     = %d\n",glaccred));
	    FXTRACE((100,"  acc green   = %d\n",glaccgreen));
	    FXTRACE((100,"  acc blue    = %d\n",glaccblue));
	    FXTRACE((100,"  acc alpha   = %d\n",glaccalpha));
	    FXTRACE((100,"  stereo      = %d\n",glstereo));
	    FXTRACE((100,"  accelerated = %d\n",glaccel));

	    // May the best visual win
	    if(dmatch<bestmatch){
	      bestmatch=dmatch;
	      bestvis=i;
	      }
	    }
	  }
	}

    // Still no luck
    if(bestvis<0){ fxerror("%s::create: no OpenGL visual available.\n",getClassName()); }

    // Report best visual
    FXTRACE((100,"Best Pixel Format (%d) match value = %d\n",bestvis,bestmatch));
    
    // Get the true h/w capabilities
    pixelformat=bestvis;
    FXMALLOC(&info,PIXELFORMATDESCRIPTOR,1);
    ((PIXELFORMATDESCRIPTOR*)info)->nSize=sizeof(PIXELFORMATDESCRIPTOR);
    ((PIXELFORMATDESCRIPTOR*)info)->nVersion=1;
    DescribePixelFormat(hdc,pixelformat,sizeof(PIXELFORMATDESCRIPTOR),(PIXELFORMATDESCRIPTOR*)info);
    
    // Make a palette for it if needed
    if(((PIXELFORMATDESCRIPTOR*)info)->dwFlags&PFD_NEED_PALETTE){
      makeOpenGLPalette();
      }
    FXTRACE((100,"System Palette: %d\n",(((PIXELFORMATDESCRIPTOR*)info)->dwFlags&PFD_NEED_SYSTEM_PALETTE)));
    
    // Done with that window
    if(flags&VISUAL_IMAGE){
      ::SelectObject(hdc,hOldBm);
      ::DeleteDC((HDC)hdc);
      ::DeleteObject(hbm);
      }
    else {
      ::ReleaseDC(GetDesktopWindow(),hdc);
      }

    
//////////////////// STILL MISSING: SETUP FOR PALETTE, NUMCOLOR ETC ////////////////
    
    // Report what we got if tracing enabled
    FXTRACE((100,"  redSize:        %d\n",getActualRedSize()));
    FXTRACE((100,"  greenSize:      %d\n",getActualGreenSize()));
    FXTRACE((100,"  blueSize:       %d\n",getActualBlueSize()));
    FXTRACE((100,"  alphaSize:      %d\n",getActualAlphaSize()));
    FXTRACE((100,"  depthSize:      %d\n",getActualDepthSize()));
    FXTRACE((100,"  stencilSize:    %d\n",getActualStencilSize()));
    FXTRACE((100,"  accumRedSize:   %d\n",getActualAccumRedSize()));
    FXTRACE((100,"  accumGreenSize: %d\n",getActualAccumGreenSize()));
    FXTRACE((100,"  accumBlueSize:  %d\n",getActualAccumBlueSize()));
    FXTRACE((100,"  accumAlphaSize: %d\n",getActualAccumAlphaSize()));
    FXTRACE((100,"  doubleBuffer:   %d\n",isDoubleBuffer()));
    FXTRACE((100,"  stereo:         %d\n",isStereo()));
    FXTRACE((100,"  accelerated:    %d\n",isAccelerated()));

#endif

    if(flags&VISUAL_OVERLAY){
      overlay = new FXOVVisual(this);
      overlay->create();
      if(0==overlay->getActualOverlaySize()){
        delete overlay;
        overlay=NULL;
        }

      FXTRACE((100,"  overlay:        %d\n",getActualOverlaySize()));
      }
    
#endif
    
    // Prevent subsequent initializations
#ifndef WIN32
    xid=1;
#else
    xid=(void*)1;
#endif
    }
  }


#ifndef WIN32
FXint FXGLVisualEx::getActualRedSize() const {
  if(!(flags&VISUAL_OSMESA)) return FXGLVisual::getActualRedSize();
  else return redSize;
  }

FXint FXGLVisualEx::getActualGreenSize() const {
  if(!(flags&VISUAL_OSMESA)) return FXGLVisual::getActualGreenSize();
  else return greenSize;
  }

FXint FXGLVisualEx::getActualBlueSize() const {
  if(!(flags&VISUAL_OSMESA)) return FXGLVisual::getActualBlueSize();
  else return blueSize;
  }

FXint FXGLVisualEx::getActualAlphaSize() const {
  if(!(flags&VISUAL_OSMESA)) return FXGLVisual::getActualAlphaSize();
  else return alphaSize;
  }

FXint FXGLVisualEx::getActualDepthSize() const {
  if(!(flags&VISUAL_OSMESA)) return FXGLVisual::getActualDepthSize();
  else return depth;
  }

FXint FXGLVisualEx::getActualStencilSize() const {
  if(!(flags&VISUAL_OSMESA)) return FXGLVisual::getActualStencilSize();
  else return stencilSize;
  }

FXint FXGLVisualEx::getActualAccumRedSize() const {
  if(!(flags&VISUAL_OSMESA)) return FXGLVisual::getActualAccumRedSize();
  else return accumRedSize;
  }

FXint FXGLVisualEx::getActualAccumGreenSize() const {
  if(!(flags&VISUAL_OSMESA)) return FXGLVisual::getActualAccumGreenSize();
  else return accumGreenSize;
  }

FXint FXGLVisualEx::getActualAccumBlueSize() const {
  if(!(flags&VISUAL_OSMESA)) return FXGLVisual::getActualAccumBlueSize();
  else return accumBlueSize;
  }

FXint FXGLVisualEx::getActualAccumAlphaSize() const {
  if(!(flags&VISUAL_OSMESA)) return FXGLVisual::getActualAccumAlphaSize();
  else return accumAlphaSize;
  }

FXbool FXGLVisualEx::isDoubleBuffer() const {
  if(!(flags&VISUAL_OSMESA)) return FXGLVisual::isDoubleBuffer();
  else return false;
  }

FXbool FXGLVisualEx::isStereo() const {
  if(!(flags&VISUAL_OSMESA)) return FXGLVisual::isStereo();
  else return false;
  }

FXbool FXGLVisualEx::isAccelerated() const {
  if(!(flags&VISUAL_OSMESA)) return FXGLVisual::isAccelerated();
  else return false;
  }
#endif


// Get actual overlay size
FXint FXGLVisualEx::getActualOverlaySize() const {
#ifndef WIN32
  if(!(flags&VISUAL_OSMESA))
#endif
  {if(!info){ fxerror("%s::getActualOverlaySize: visual not yet initialized.\n",getClassName()); }}
  return overlay?overlay->getActualOverlaySize():0;
  }


// Save to stream
void FXGLVisualEx::save(FXStream& store) const {
  FXGLVisual::save(store);
  store << overlaySize;
  }


// Load from stream
void FXGLVisualEx::load(FXStream& store){ 
  FXGLVisual::load(store);
  store >> overlaySize;
  }
  
// Detach visual
void FXGLVisualEx::detach(){
  if(xid){
    FXTRACE((100,"%s::detach %08x\n",getClassName(),this));
    if(overlay){overlay->detach();}
    FXGLVisual::detach();
    }
  }

// Destroy visual
void FXGLVisualEx::destroy(){
  if(xid){
    FXTRACE((100,"%s::destroy %08x\n",getClassName(),this));
    if(overlay){overlay->destroy();}
    FXGLVisual::destroy();
    }
  }

// Destroy
FXGLVisualEx::~FXGLVisualEx(){
  FXTRACE((100,"%s::~%s %08x\n",getClassName(),getClassName(),this));
  destroy();
  if(overlay){delete overlay;}
  }


