/********************************************************************************
*                                                                               *
*                            V i s u a l E x  C l a s s                         *
*                                                                               *
*********************************************************************************
* Copyright (C) 2003 by ABAQUS, Inc.   All Rights Reserved.                     *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Library General Public                   *
* License as published by the Free Software Foundation; either                  *
* version 2 of the License, or (at your option) any later version.              *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Library General Public License for more details.                              *
*                                                                               *
* You should have received a copy of the GNU Library General Public             *
* License along with this library; if not, write to the Free                    *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
********************************************************************************/
#ifndef FXGLVISUALEX_H
#define FXGLVISUALEX_H
// begin local includes
#include <fgl_FGLAPI.h>
#include <FXGLVisual.h>

/// Visual describes pixel format of a drawable
class FXOVVisual; // CAE
class FXDCWindow; // CAE
class FGLAPI FXGLVisualEx : public FXGLVisual {
  FXDECLARE(FXGLVisualEx)
  friend class FXWindow;
  friend class FXImage;
  friend class FXBitmap;
  friend class FXOVVisual;
  friend class FXDCWindow;
  friend class FXGLDC;
protected:
  FXint        overlaySize;	    // Desired #bits for overlay
protected:
  FXGLVisualEx();
  FXOVVisual   *overlay;	    // Overlay visual, if any
#ifndef WIN32
  void setuposmesacolor();
#endif
private:
  FXGLVisualEx(const FXGLVisualEx&);
  FXGLVisualEx &operator=(const FXGLVisualEx&);
public:

  /// Construct default visual
  FXGLVisualEx(FXApp* a,FXuint flags, FXint userVisualId = 0);

  /// Create visual
  virtual void create();
  
  /// Detach visual
  virtual void detach();

  /// Destroy visual
  virtual void destroy();

  /// Get sizes for bit-planes
  FXOVVisual *getOverlayVisual() const { return overlay; }

  /// Get sizes for bit-planes
  FXint getOverlaySize() const { return overlaySize; }

  /// Set sizes for bit-planes
  void setOverlaySize(FXint os){ overlaySize=os; }

#ifndef WIN32
// Begin: Only overridden to support OSMesa
  /// Get ACTUAL sizes for bit-planes
  FXint getActualRedSize() const;
  FXint getActualGreenSize() const;
  FXint getActualBlueSize() const;
  FXint getActualAlphaSize() const;
  FXint getActualDepthSize() const;
  FXint getActualStencilSize() const;
  FXint getActualAccumRedSize() const;
  FXint getActualAccumGreenSize() const;
  FXint getActualAccumBlueSize() const;
  FXint getActualAccumAlphaSize() const;

  /// Is it double buffered?
  FXbool isDoubleBuffer() const;

  /// Is it stereo?
  FXbool isStereo() const;

  /// Is it hardware-accelerated?
  FXbool isAccelerated() const;
// End: Only overridden to support OSMesa
#endif

  /// Get ACTUAL sizes for bit-planes
  FXint getActualOverlaySize() const;

  /// Save visual info to a stream
  virtual void save(FXStream& store) const;

  /// Load visual info to a stream
  virtual void load(FXStream& store);

  /// Destructor
  virtual ~FXGLVisualEx();
  
#ifndef WIN32
  FXuint getVisualId() { return visual ? ((Visual *)visual)->visualid : 0; }
#endif
  };
  

#endif
