// begin local includes
/********************************************************************************
*                                                                               *
*                    O v e r l a y   V i s u a l   C l a s s                    *
*                                                                               *
*********************************************************************************
* Copyright (C) 2003 by ABAQUS, Inc.   All Rights Reserved.                     *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Library General Public                   *
* License as published by the Free Software Foundation; either                  *
* version 2 of the License, or (at your option) any later version.              *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Library General Public License for more details.                              *
*                                                                               *
* You should have received a copy of the GNU Library General Public             *
* License along with this library; if not, write to the Free                    *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
********************************************************************************/
#define FGLDLL_EXPORTS
#include <xincs.h>
#include <fxver.h>
#include <fxdefs.h>
#include <fxkeys.h>
#include <FXStream.h>
#include <FXString.h>
#include <FXObject.h>
#include <FXDict.h>
#include <FXRegistry.h>
#include <FXAccelTable.h>
#include <FXObjectList.h>
#include <FXApp.h>
#include <FXId.h>
#include <FXVisual.h>
#include <FXOVVisual.h>
#include <FXGLVisual.h>
#include <FXGLVisualEx.h>

// end local includes

#if defined (ABQ_LINUX)
// must explicitly include Xmd.h since glx.h on linux does not
#include <X11/Xmd.h>
#endif

/*******************************************************************************/

#ifndef WIN32

#define DISPLAY(app) ((Display*)((app)->getDisplay()))

/* SERVER_OVERLAY_VISUALS property element */
typedef struct _OverlayInfo
{
    CARD32  overlay_visual;
    CARD32  transparent_type;
    CARD32  value;
    INT32   layer;
} OverlayInfo;

void  *FXOVVisual::serverOverlayInfo = NULL;
int    FXOVVisual::numOverlays = 0;

#endif // ! WIN32


// Maximum size of the colormap; for high-end graphics systems
// you may want to define HIGHENDGRAPHICS to allow large colormaps
#ifdef HIGHENDGRAPHICS
#define MAX_MAPSIZE 4096
#else 
#define MAX_MAPSIZE 256
#endif


/*******************************************************************************/
     
     
// Object implementation
FXIMPLEMENT(FXOVVisual,FXId,NULL,0)



// Deserialization
FXOVVisual::FXOVVisual(){
  FXTRACE((100,"FXOVVisual::FXOVVisual %08x INVALID\n",this));
  overlaySize=0;
  actualOverlaySize=0;
  info=NULL;
  clearPixel=0;
  overlayColors=NULL;
  lastColor=0;
#ifndef WIN32
  visual=NULL;
  colormap=0;
  gc=0;
  pixelsStart=256;
  pixelsEnd=0;
  serverOverlayInfo=NULL;
  numOverlays=0;
#endif
  }


// Construct 
FXOVVisual::FXOVVisual(FXVisual *base,FXuint reqDepth):FXId(base->getApp()){
  FXTRACE((100,"FXOVVisual::FXOVVisual %08x\n",this));
  parent=base;
  overlaySize=reqDepth;
  actualOverlaySize=0;
  info=NULL;
  clearPixel=0;
  overlayColors=NULL;
  lastColor=0;
#ifndef WIN32
  visual=NULL;
  colormap=0;
  gc=0;
  displayIsSun=false;
  pixelsStart=256;
  pixelsEnd=0;
  serverOverlayInfo=NULL;
  numOverlays=0;
#endif
  }


/*******************************************************************************/

// Manage overlay palette

void FXOVVisual::setOverlayPalette(FXint numColors, FXColor *colors, FXID xid_in){

  FXTRACE((100,"FXOVVisual::setOverlayPalette(%d) %0x8\n",numColors,this));

  if( 0 < numColors && NULL != colors ){
    if( numColors > actualOverlaySize )
	numColors = actualOverlaySize;
    if( NULL==overlayColors )
      FXCALLOC(&overlayColors,FXColor,actualOverlaySize);

#ifndef WIN32
    Display *display = DISPLAY(getApp());

    // Must be handled synchronously if displaying to Sun
    static bool bSynchronize = (NULL != getenv("HKS_XSYNCHRONIZE"));
    XSync(display, FALSE);
    if(!bSynchronize) XSynchronize(display, TRUE);

    // Release previous palette
    if( 0 < lastColor && NULL!=overlayColors ){
      unsigned long *allocatedColors, i;
      int nColors = 0;
      FXColor *ptrColor = overlayColors;
      FXTRACE((100,"FXOVVisual::setOverlayPalette: Releasing old colors\n"));

      FXMALLOC(&allocatedColors,unsigned long,lastColor);
      for(i=0; i < lastColor; i++, ptrColor++){
	if( FXALPHAVAL(*ptrColor) ){
	  allocatedColors[nColors++] = i;
	  *ptrColor = 0;
	  }
        }
      if(pixelsStart<pixelsEnd){
        XFreeColors( display, (Colormap)colormap, 
		     pixelsBuffer+pixelsStart, pixelsEnd-pixelsStart, 0UL );
        XFreeColors( display, DefaultColormap(display, DefaultScreen(display)), 
		     allocatedColors, nColors, 0UL );
	pixelsStart = 256;
	pixelsEnd = 0;
        }
      else{
        XFreeColors( display, (Colormap)colormap, 
		     allocatedColors, nColors, 0UL );
        }
      FXFREE(&allocatedColors);
      }
      
    // Allocate entries for new palette
    lastColor = actualOverlaySize;
    FXColor *ptrColor = colors;

//-----------------------------------
    // In this block of code we try to use existing colors in the default
    // color map.  This reduces flahing when the colormaps are managed.
    XVisualInfo *ovi = (XVisualInfo*)info;
    int nCells = (!displayIsSun) ? ovi->colormap_size - 1 : 0;
    if(!displayIsSun){
      Status allocedcolor = 1;
      int screen = DefaultScreen(display);
      int nLastPixel = 0, nFirstPixel = 256;
      XVisualInfo *ovi = (XVisualInfo *)info;
      if(colormap != DefaultColormap(display, screen) &&
          (ovi->depth <= 8) && (DefaultDepth(display, screen) == ovi->depth) &&
	  XAllocColorCells(display,(Colormap)colormap,True,0,0,pixelsBuffer,nCells)){

	for( FXint i=0; i < numColors && allocedcolor; i++, ptrColor++ ){
	  XColor clr, xcolor;
	  // Make sure ALPHA is solid.  (Used as a flag for !WIN32 logic)
	  FXColor color = FXRGBA(FXREDVAL(*ptrColor),FXGREENVAL(*ptrColor),FXBLUEVAL(*ptrColor),0xFF);

	  // We try to get requested colors
	  xcolor.red   = FXREDVAL(color) * 255;
	  xcolor.green = FXGREENVAL(color) * 255;
	  xcolor.blue  = FXBLUEVAL(color) * 255;
	  xcolor.flags = DoRed|DoGreen|DoBlue;

	  allocedcolor = XAllocColor(display, DefaultColormap(display, screen), &xcolor);
	  if(allocedcolor){
	    // Use same pixel value as cell we got for default cmap if it fits
	    clr=xcolor;
	    allocedcolor = XStoreColor(display,(Colormap)colormap,&clr);
	    if(allocedcolor){
              FXTRACE((100,"FXOVVisual::setOverlayPalette: Stored color (0x%08x) %d\n",color,clr.pixel));
	      if(clr.pixel > nLastPixel)  nLastPixel  = clr.pixel;
	      if(clr.pixel < nFirstPixel) nFirstPixel = clr.pixel;
	      overlayColors[clr.pixel] = color;
	      }
	    }
	  }

	if(allocedcolor && nLastPixel<nCells){
	  // Free remaining cells to reduce flashing
	  if(0!=nFirstPixel)
	    XFreeColors(display, colormap, pixelsBuffer, nFirstPixel, 0);
	  XFreeColors(display, colormap, pixelsBuffer+nLastPixel, nCells-nLastPixel, 0);
	  pixelsStart = nFirstPixel;
	  pixelsEnd   = nLastPixel;
	  }
	else{
          FXTRACE((100,"FXOVVisual::setOverlayPalette: Abandoning stored colors.\n"));
	  XFreeColors(display, colormap, pixelsBuffer, nCells, 0);
	  nCells = 0;
	  }
        }
      else
        nCells = 0;
      }

//-----------------------------------

    // If we couldn't borrow from the default color map, allocate colors normally.
    if(!nCells){
      ptrColor = colors;
      for( FXint i=0; i < numColors; i++, ptrColor++ )
        getOverlayPixel(*ptrColor,0);
      }

    // Return to normal processing (see Sun comment above).
    if(!bSynchronize) XSynchronize(display, FALSE);
#else
    for(int i=0, j=0; i < actualOverlaySize && j < numColors; i++){
      if( i != clearPixel )
	overlayColors[i] = colors[j++];
      }
    if( lastColor < numColors )
      lastColor = numColors;
    if( clearPixel < numColors )
      numColors++;

    // If xid_in was passed in, go ahead and realize the palette
    if( xid_in ){
      FXASSERT(sizeof(FXColor)==sizeof(COLORREF));
      HDC hdc=::GetDC((HWND)xid_in);
      if( hdc ){
	wglSetLayerPaletteEntries(hdc, 1, 0, lastColor, (COLORREF*)overlayColors);
	wglRealizeLayerPalette(hdc, 1, TRUE);
	::ReleaseDC((HWND)xid_in,hdc);
	}
      }
#endif
    }
  }

FXPixel FXOVVisual::getOverlayPixel(FXColor colorIn, FXID xid_in){
  FXint idx;
  FXColor color, *ptrColor;
  FXASSERT(actualOverlaySize);

  FXTRACE((100,"FXOVVisual::getOverlayPixel(0x%08x) %0x8\n",colorIn, this));

#ifndef WIN32
  // Make sure ALPHA is solid.  (Used as a flag for !WIN32 logic)
  color = FXRGBA(FXREDVAL(colorIn),FXGREENVAL(colorIn),FXBLUEVAL(colorIn),0xFF);
#else
  color = RGB(FXREDVAL(colorIn),FXGREENVAL(colorIn),FXBLUEVAL(colorIn));
#endif

  // Allocate palette if needed
  if( NULL==overlayColors ){
    FXCALLOC(&overlayColors,FXColor,actualOverlaySize);
#ifndef WIN32
    lastColor = actualOverlaySize;
#endif
    }
  else{
    // Look for color in palette
    ptrColor = overlayColors;
    for( idx=0; idx<lastColor; idx++, ptrColor++ ){
      if( *ptrColor == color && ((FXPixel)idx)!=clearPixel ){
        FXTRACE((100,"FXOVVisual::getOverlayPixel(0x%08x) = %d\n",color, idx));
        return (FXPixel)idx;
        }
      }
    }

#ifndef WIN32
  // Add color to palette if there is room.  
  Status allocedcolor;
  XColor xcolor;
  Display *display = DISPLAY(getApp());

  // We try to get requested colors
  xcolor.red   = FXREDVAL(color) * 255;
  xcolor.green = FXGREENVAL(color) * 255;
  xcolor.blue  = FXBLUEVAL(color) * 255;
  xcolor.flags = DoRed|DoGreen|DoBlue;

  // First try just using XAllocColor
  allocedcolor=XAllocColor(display,(Colormap)colormap,&xcolor);

  // Otherwise try finding best match
  if(!allocedcolor){
    // Get colors in the map
    FXuint bestmatch,i;
    FXdouble mindist,dist,dr,dg,db;
    XColor table[256];
    FXuint mapsize=((Visual*)visual)->map_entries;
    if(mapsize>256) mapsize=256;
    for(i=0; i<mapsize; i++){
      table[i].pixel=i;
      table[i].flags=DoRed|DoGreen|DoBlue;
      }
    XQueryColors(display,colormap,table,mapsize);

    // Find best match
    for(mindist=1.0E10,bestmatch=0,i=0; i<mapsize; i++){
      dr=xcolor.red-table[i].red;
      dg=xcolor.green-table[i].green;
      db=xcolor.blue-table[i].blue;
      dist=dr*dr+dg*dg+db*db;
      if(dist<mindist){
        bestmatch=i;
        mindist=dist;
        if(mindist==0.0) break;
        }
      }

    // Return result
    xcolor.red=table[bestmatch].red;
    xcolor.green=table[bestmatch].green;
    xcolor.blue=table[bestmatch].blue;

    // Try to allocate the closest match color.  This should only
    // fail if the cell is read/write.  Otherwise, we're incrementing
    // the cell's reference count.
    allocedcolor=XAllocColor(display,colormap,&xcolor);
    FXTRACE((100,"FXOVVisual::getOverlayPixel: Matched color (0x%08x) %d\n",
	FXRGB((FXuchar)(xcolor.red/255.f),(FXuchar)(xcolor.green/255.f),(FXuchar)(xcolor.blue/255.f)),
	xcolor.pixel));

    // Cell was read/write; we can't use read/write cells as some 
    // other app might change our colors and mess up the display.
    // However, rumor has it that some X terminals and the Solaris 
    // X server have XAllocColor fail even if we're asking for a 
    // color which is known to be in the table; so we'll use this 
    // color anyway and hope nobody changes it..
    if(!allocedcolor){
      allocedcolor=TRUE;
      xcolor.pixel=bestmatch;
      }
    }

  if(allocedcolor){
    overlayColors[xcolor.pixel] = color;
    return xcolor.pixel;
    }

#else

  // Add color to palette if there is room.  
  if( lastColor<actualOverlaySize ){
    // Skip over the index for the clear color.  
    if( ((FXPixel)lastColor) == clearPixel ){
      if( (1+lastColor) >= actualOverlaySize )
	return (0==clearPixel) ? 1 : 0;
      lastColor++;
      }
    overlayColors[lastColor] = color;
    lastColor++;

    if( xid_in ){
      HDC hdc=::GetDC((HWND)xid_in);
      if( hdc ){
        FXASSERT(sizeof(FXColor)==sizeof(COLORREF));
	wglSetLayerPaletteEntries(hdc, 1, 0, lastColor, (COLORREF*)overlayColors);
	wglRealizeLayerPalette(hdc, 1, TRUE);
	::ReleaseDC((HWND)xid_in,hdc);
        }
      }

    return lastColor-1;
    }
#endif

  // If palette is full, return the first color.
  return (0==clearPixel) ? 1 : 0;
  }  


/*******************************************************************************/


// Initialize
void FXOVVisual::create(){
  if(!xid){
    FXTRACE((100,"FXOVVisual::create  %0x8\n", this));
    
#ifndef WIN32

    // Get the overlay visual
    if(overlaySize>0){
      Display *display = DISPLAY(getApp());
      int screen = DefaultScreen(display);
      Window root = RootWindow(display, screen);
      XVisualInfo ovi_data;
      XVisualInfo *ovi=0;

      const char * psVendor = ServerVendor(display);
      FXbool bToIBM = (psVendor  &&  ((!strncmp("International Business Machines",psVendor,31))  ||
	  (!strncmp("etnI",psVendor,4)) ) );  // For COMPAQ to IBM
      displayIsSun = (psVendor  &&  (!strncmp("Sun ",psVendor,4)) );
      
#ifdef HAVE_OPENGL
      int anOverlayPlaneAttribs[] =
	{
	    GLX_BUFFER_SIZE, overlaySize,
	    GLX_LEVEL, 1,
#ifdef GLX_TRANSPARENT_TYPE_EXT
	    GLX_TRANSPARENT_TYPE_EXT, GLX_TRANSPARENT_INDEX_EXT,
#endif
	    0
	};

      ovi = glXChooseVisual( display, screen, anOverlayPlaneAttribs );
#ifdef GLX_TRANSPARENT_TYPE_EXT
      if(ovi && anOverlayPlaneAttribs[4]){
	int transIndex;
        if(0==glXGetConfig(display,ovi,GLX_TRANSPARENT_INDEX_VALUE_EXT,&transIndex)){
	  clearPixel = transIndex;
	  actualOverlaySize = (1 << ovi->depth);  // Only set AFTER 'clear' established
          FXTRACE((100,"FXOVVisual::create - visual: %x  clearPixel: %d  size: %d\n", ovi->visualid, clearPixel, actualOverlaySize));
	  }
	else{
	  anOverlayPlaneAttribs[4] = 0;
	  ovi = glXChooseVisual( display, screen, anOverlayPlaneAttribs );
	  }
        }
      else if(anOverlayPlaneAttribs[4]){
	anOverlayPlaneAttribs[4] = 0;
	ovi = glXChooseVisual( display, screen, anOverlayPlaneAttribs );
	}
#endif
#else
      FXint nvi,i,d,dbest;
      XVisualInfo vitemplate;
      XVisualInfo *vi;
      vitemplate.screen=screen;
      vitemplate.class=PseudoColor;
      vi=XGetVisualInfo(DISPLAY(getApp()),VisualScreenMask|VisualClassMask,&vitemplate,&nvi);
      if(!vi){
        vitemplate.class=StaticColor;
        vi=XGetVisualInfo(DISPLAY(getApp()),VisualScreenMask|VisualClassMask,&vitemplate,&nvi);
        }
      if(vi){
        for(i=0,dbest=1000000; i<nvi; i++){
          d=vi[i].depth-overlaySize;
          if(d<0) d*=-100;     
          if(d<dbest){
            dbest=d;
	    ovi = &ovi_data;
	    ovi_data = vi[i];
            }
          }
        FXTRACE((100,"FXOVVisual::create - trying Non-OpenGL visual: %x\n", ovi->visualid));
        XFree((char*)vi);
        }
#endif
      if(NULL == ovi){
        FXTRACE((100,"FXOVVisual::create - Could not find a usable visual\n"));
	return;
        }

      // Get the value of the transparent pixel
      if( 0 == actualOverlaySize ){
	// Get overlay info from the server
	if(NULL==serverOverlayInfo && -1!=numOverlays){
	  Atom overlayVisualsAtom = XInternAtom(display, "SERVER_OVERLAY_VISUALS", TRUE);
	  if (overlayVisualsAtom != None){
            Atom actualType;
            unsigned long sizeData, bytesLeft;
            int actualFormat;
	    Status status = XGetWindowProperty(display, root, overlayVisualsAtom,
                                     0L, 10000L, FALSE, overlayVisualsAtom,
                                     &actualType, &actualFormat, &sizeData, &bytesLeft,
                                     (unsigned char **) &serverOverlayInfo);
            if( status != Success || actualType != overlayVisualsAtom ||
		actualFormat != 32 || sizeData < 4)
              numOverlays = -1;  // So we will not try (and fail) again
	    else
	      numOverlays = sizeData/(sizeof(OverlayInfo)/4);
	    }
	  if(0>=numOverlays){
	    XFree(ovi);
	    if(NULL!=serverOverlayInfo){
	      XFree(serverOverlayInfo);
	      serverOverlayInfo = NULL;
	      }
	    FXTRACE((100,"FXOVVisual::create - Could not find a visual with usable properties\n"));
	    return;
	    }
	  }
	if(NULL!=serverOverlayInfo && 0<numOverlays){
	  OverlayInfo *serverOverlay = (OverlayInfo *) serverOverlayInfo;
	  for(int j=0; j<numOverlays; j++, serverOverlay++){
	    if(ovi->visualid == serverOverlay->overlay_visual){
	      if(0<serverOverlay->layer){
		if(None!=serverOverlay->transparent_type){
	          clearPixel = serverOverlay->value;
	          actualOverlaySize = (1 << ovi->depth);  // Only set AFTER 'clear' established
                  FXTRACE((100,"FXOVVisual::create - visual: %d  clearPixel from Atom: %d  size: %d\n", ovi->visualid, clearPixel,actualOverlaySize));
		  }
		else if(!bToIBM){
	          clearPixel = 0;
	          actualOverlaySize = (1 << ovi->depth);  // Only set AFTER 'clear' established
                  FXTRACE((100,"FXOVVisual::create - visual: %d  assumed clearPixel from Atom: %d  size: %d\n", ovi->visualid, clearPixel,actualOverlaySize));
		  }
	        }
	      break;
	      }
	    }
	  }

        // If we still don't have a clearPixel then we can't get overlay
        if( 0 == actualOverlaySize ){
	  if(ovi!=&ovi_data){XFree(ovi);}
	  FXTRACE((100,"FXOVVisual::create - Could not determine clear pixel for any visual\n"));
	  return;
          }
        }

      // Get overlay colormap (but we do not need to allocate colors yet)
      FXASSERT(None==colormap);
      if(displayIsSun && ovi->visual==DefaultVisual(display, screen))
        colormap = DefaultColormap(display, screen);
      if(None==colormap){
        // Create overlay colormap (but we do not need to allocate colors yet)
        colormap = XCreateColormap(display, root, ovi->visual, AllocNone);
        if(None == colormap){
	  XFree(ovi);
	  return;
	  }
	else
    	  XInstallColormap(display,colormap);
        }

#if 0
      // Make a GC for the overlay visual
      FXID drawable=XCreatePixmap(display,root,1,1,ovi->depth);
      gc=XCreateGC(display,drawable,0x0,0);
      XFreePixmap(display,drawable);
#endif

      // Keep overlay info for later
      visual = ovi->visual;
      FXMALLOC(&info,XVisualInfo,1);
      memcpy(info,ovi,sizeof(XVisualInfo));
      XFree(ovi);

      // What we matched
      FXTRACE((100,"OPENGL Overlay Visual = 0x%02x\n",((Visual*)visual)->visualid));
      FXTRACE((100,"  overlaySize:    %d\n",getActualOverlaySize()));
      FXTRACE((100,"  colormap:       %x %s\n",colormap,
	    colormap == DefaultColormap(display, screen) ? "(Default)" : ""));
      }

#else	// WIN32

    // Overlay is only available on Windows though OpenGL (or Direct 3D?)
#ifdef HAVE_OPENGL

    // Get some window handle
    HDC hdc=GetDC(GetDesktopWindow());

    // Get any overlay capabilities only if overlay requested
    FXGLVisualEx *parentEx=(FXGLVisualEx*)parent;
    if((0<parentEx->overlaySize)&&(((PIXELFORMATDESCRIPTOR*)parentEx->info)->bReserved & 0x7)){
      FXMALLOC(&info,LAYERPLANEDESCRIPTOR,1);
      LAYERPLANEDESCRIPTOR *pLPD = (LAYERPLANEDESCRIPTOR*)info;
      pLPD->nSize=sizeof(LAYERPLANEDESCRIPTOR);
      pLPD->nVersion=1;
      if (wglDescribeLayerPlane(hdc, parentEx->pixelformat, 1, pLPD->nSize, pLPD)){
	if (LPD_TYPE_COLORINDEX == pLPD->iPixelType){
	  actualOverlaySize = (1 << pLPD->cColorBits);
	  if( LPD_TRANSPARENT & pLPD->dwFlags )
	    clearPixel = (FXint) (pLPD->crTransparent);
	  }
	else{
	  FXFREE(&info);
          }
        }
      }
    FXTRACE((100,"System Overlay Palette: %d\n",actualOverlaySize ));
    
    // Done with that window
    ReleaseDC(GetDesktopWindow(),hdc);
   
    // Report what we got if tracing enabled
    FXTRACE((100,"  overlaySize:    %d\n",getActualOverlaySize()));
    
#endif // OPENGL
#endif // WIN32
    
    // Prevent subsequent initializations
    xid=(FXID)1;
    }
  }


/*******************************************************************************/


// Save to stream
void FXOVVisual::save(FXStream& store) const {
  FXId::save(store);
  store << overlaySize;
  }


// Load from stream
void FXOVVisual::load(FXStream& store){ 
  FXId::load(store);
  store >> overlaySize;
  }

// Detach visual
void FXOVVisual::detach(){
  if(xid){
    FXTRACE((100,"FXOVVisual::detach %08x\n",this));
    xid = 0;
    }
  }

// Destroy visual
void FXOVVisual::destroy(){
  if(xid){
    FXTRACE((100,"FXOVVisual::destroy %08x\n",this));
#ifndef WIN32
    Display *display = DISPLAY(getApp());
    if(0!=gc){XFreeGC(display,(GC)gc);}
    if(None!=colormap){
      if( 0 < lastColor && NULL!=overlayColors ){
        unsigned long *allocatedColors, i;
        int numColors = 0;
        FXColor *ptrColor = overlayColors;
        FXMALLOC(&allocatedColors,unsigned long,lastColor);
        for(i=0; i < lastColor; i++, ptrColor++){
          if( FXALPHAVAL(*ptrColor) )
	    allocatedColors[numColors++] = i;
          }
        XFreeColors( display, (Colormap)colormap, 
		     allocatedColors, numColors, 0UL );
        FXFREE(&allocatedColors);
        lastColor = 0;
        }
      if(colormap != DefaultColormap(display, DefaultScreen(display)))
        XFreeColormap(display, (Colormap)colormap);
      }
    if(NULL!=serverOverlayInfo) XFree(serverOverlayInfo);

    visual=NULL;
    colormap=0;
    gc=0;
    serverOverlayInfo=NULL;
    numOverlays=0;
#endif
    if(info) FXFREE(&info);
    if(overlayColors) FXFREE(&overlayColors);

    actualOverlaySize=0;
    info=NULL;
    clearPixel=0;
    overlayColors=NULL;
    lastColor=0;
    xid=0;
    }
  }


// Destroy
FXOVVisual::~FXOVVisual(){
  FXTRACE((100,"FXOVVisual::~FXOVVisual %08x\n",this));
  if(xid)
    destroy();
  }
