/********************************************************************************
*                                                                               *
*                    O v e r l a y   V i s u a l   C l a s s                    *
*                                                                               *
*********************************************************************************
* Copyright (C) 2003 by ABAQUS, Inc.   All Rights Reserved.                     *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Library General Public                   *
* License as published by the Free Software Foundation; either                  *
* version 2 of the License, or (at your option) any later version.              *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Library General Public License for more details.                              *
*                                                                               *
* You should have received a copy of the GNU Library General Public             *
* License along with this library; if not, write to the Free                    *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
********************************************************************************/
#ifndef FXOVVISUAL_H
#define FXOVVISUAL_H
// begin local includes
#include <fgl_FGLAPI.h>
#include <FXVisual.h>



/// Visual describes pixel format of a drawable
class FGLAPI FXOVVisual : public FXId {
  FXDECLARE(FXOVVisual)
#ifndef WIN32
  friend class FXGLContextEx;
  friend class FXGLDC;
#endif
protected:
  FXint         overlaySize;         // Desired #bits for overlay
  FXint         actualOverlaySize;   // Actual #bits for overlay
#ifndef WIN32
  void         *visual;              // Application visual [Visual]
  void         *info;                // XVisualInfo, if any
  FXID          colormap;            // Colormap, if any
  void         *gc;                  // Drawing GC
  bool		displayIsSun;        // True is X Server is a Sun machine
  unsigned long pixelsBuffer[256];
  unsigned long pixelsStart;
  unsigned long pixelsEnd;
  static void  *serverOverlayInfo;
  static int    numOverlays;
#else
  void         *info;                // LAYERPLANEDESCRIPTOR, if any
#endif
  FXPixel       clearPixel;          // Index of clear/invisible palette entry
private:
  FXVisual     *parent;		     // Base visual for this overlay visual
  FXint         lastColor;           // Number of entries in the palette
  FXColor      *overlayColors;	     // Array of colors

protected:
  FXOVVisual();
private:
  FXOVVisual(const FXOVVisual&);
  FXOVVisual &operator=(const FXOVVisual&);

public:
  
  /// Construct default overlay visual
  FXOVVisual(FXVisual *,FXuint reqDepth=2);

  /// Create overlay visual
  virtual void create();
  
  /// Detach overlay visual
  virtual void detach();

  /// Destroy overlay visual
  virtual void destroy();

  /// Get sizes for bit-planes
  FXint getOverlaySize() const { return overlaySize; }

  /// Set sizes for bit-planes
  void setOverlaySize(FXint os){ overlaySize=os; }

  /// Get ACTUAL sizes for bit-planes
  FXint getActualOverlaySize() const { return actualOverlaySize; }

  /// Does it have overlay?
  FXbool hasOverlay() const  { return (0<actualOverlaySize); };

  /// Get device pixel value for color on overlay plane
  void setOverlayPalette(FXint numColors, FXColor *colors, FXID xid_in);

  /// Get device pixel value for color on overlay plane
  FXPixel getOverlayPixel(FXColor color, FXID xid_in);

  /// Get device pixel value for clear color on overlay plane
  FXPixel getOverlayClearPixel() { return clearPixel; }

  /// Get visual type
  FXVisualType getType() const { return VISUALTYPE_INDEX; }

  /// Save visual information to a stream
  virtual void save(FXStream& store) const;
  
  /// Load visual information from a stream
  virtual void load(FXStream& store);
  
  /// Destructor
  virtual ~FXOVVisual();

#ifndef WIN32
  FXuint getVisualId() { return visual ? ((Visual *)visual)->visualid : 0; }
#endif
  };


#endif
