// -*- Mode: C++ -*-
// Begin local includes
#include <FXTheme.h>
#include <FXWindow.h>
#include <FXDCWindow.h>
#include <FXString.h>
#include <FXImage.h>
#include <FXApp.h>
#include <fxtmschema.h>

#ifdef HKS_NT

// Theme size enums
enum THEMESIZE
{
    TS_MIN,             // minimum size
    TS_TRUE,            // size without stretching
    TS_DRAW,            // size that theme mgr will use to draw part
};

// Properties
enum {
    TMT_TEXTCOLOR = 3803
};

// XP Themes
static BOOL bThemeLibLoaded = FALSE;
static HMODULE hModThemes = NULL;

typedef HANDLE HTHEME;

typedef HRESULT(__stdcall *PFNCLOSETHEMEDATA)(
    HTHEME hTheme);
typedef HRESULT(__stdcall *PFNDRAWTHEMEBACKGROUND)(
    HTHEME hTheme, HDC hdc, int iPartId, int iStateId, const RECT *pRect,  const RECT *pClipRect);
typedef HRESULT(__stdcall *PFNDRAWTHEMEEDGE)(
    HTHEME hTheme, HDC hdc, int iPartId, int iStateId, const RECT *pDestRect, 
    UINT uEdge, UINT uFlags, const RECT *pContentRect);
typedef HRESULT(__stdcall *PFNGETTHEMEPARTSIZE)(
    HTHEME hTheme, HDC hdc, int iPartId, int iStateId, RECT *prc, 
    THEMESIZE eSize, SIZE *psz);
typedef HTHEME(__stdcall *PFNOPENTHEMEDATA)(
    HWND hwnd, LPCWSTR pszClassList);
typedef HRESULT (__stdcall *PFNGETTHEMECOLOR)(
    HTHEME hTheme, int iPartId, int iStateId, int iPropId, COLORREF *pColor);
typedef HRESULT (__stdcall *PFNGETTHEMEBACKGROUNDREGION)(
    HTHEME hTheme, HDC hdc, int iPartId, int iStateId, const RECT *pRect, HRGN *pRegion);
typedef COLORREF (__stdcall *PFNGETTHEMESYSCOLOR)(
    HTHEME hTheme, int iColorId);
typedef int (__stdcall *PFNGETTHEMESYSSIZE)(
    HTHEME hTheme, int iSizeId);
typedef BOOL (__stdcall *PFNISTHEMEACTIVE)(
    );

static PFNOPENTHEMEDATA zOpenThemeData = NULL;
static PFNDRAWTHEMEBACKGROUND zDrawThemeBackground = NULL;
static PFNDRAWTHEMEEDGE zDrawThemeEdge = NULL;
static PFNGETTHEMEPARTSIZE zGetThemePartSize = NULL;
static PFNCLOSETHEMEDATA zCloseThemeData = NULL;
static PFNGETTHEMECOLOR zGetThemeColor = NULL;
static PFNGETTHEMEBACKGROUNDREGION zGetThemeBackgroundRegion = NULL;
static PFNGETTHEMESYSCOLOR zGetThemeSysColor = NULL;
static PFNGETTHEMESYSSIZE zGetThemeSysSize = NULL;
static PFNISTHEMEACTIVE zIsThemeActive = NULL;

static void InitThemes()
{
//     if (!getenv("ABAQUS_USE_THEMES"))
//         return;

    hModThemes = LoadLibrary("UXTHEME.DLL");
    if(hModThemes)
    {
        zOpenThemeData = (PFNOPENTHEMEDATA) GetProcAddress(
            hModThemes, "OpenThemeData");
        zDrawThemeBackground = (PFNDRAWTHEMEBACKGROUND) GetProcAddress(
            hModThemes, "DrawThemeBackground");
        zDrawThemeEdge = (PFNDRAWTHEMEEDGE) GetProcAddress(
            hModThemes, "DrawThemeEdge");
        zGetThemePartSize = (PFNGETTHEMEPARTSIZE) GetProcAddress(
            hModThemes, "GetThemePartSize");
        zCloseThemeData = (PFNCLOSETHEMEDATA) GetProcAddress(
            hModThemes, "CloseThemeData");
        zGetThemeColor = (PFNGETTHEMECOLOR) GetProcAddress(
            hModThemes, "GetThemeColor");
        zGetThemeBackgroundRegion = (PFNGETTHEMEBACKGROUNDREGION) GetProcAddress(
            hModThemes, "GetThemeBackgroundRegion");
        zGetThemeSysColor = (PFNGETTHEMESYSCOLOR) GetProcAddress(
            hModThemes, "GetThemeSysColor");
        zGetThemeSysSize = (PFNGETTHEMESYSSIZE) GetProcAddress(
            hModThemes, "GetThemeSysSize");
        zIsThemeActive = (PFNISTHEMEACTIVE) GetProcAddress(
            hModThemes, "IsThemeActive");

        if(zOpenThemeData 
           && zDrawThemeBackground
           && zDrawThemeEdge
           && zGetThemePartSize
           && zCloseThemeData
           && zGetThemeColor
           && zGetThemeBackgroundRegion
           && zGetThemeSysColor
           && zGetThemeSysSize
           && zIsThemeActive)
        {
            bThemeLibLoaded = TRUE;
        }
        else
        {
            FreeLibrary(hModThemes);
            hModThemes = NULL;
        }
    }
}

static void FinThemes()
{
    if (bThemeLibLoaded) {
        FreeLibrary(hModThemes);
        hModThemes = NULL;
        bThemeLibLoaded = FALSE;
    }
}

#endif

FXTheme::FXTheme( FXWindow *window, FXuint cls )
    : m_theme(NULL),
      m_window(window),
      m_class(THEME_NONE)
{
    reset( cls );
}

FXTheme::FXTheme( FXWindow *window )
    : m_theme(NULL),
      m_window(window),
      m_class(THEME_NONE)
{
}

FXTheme::~FXTheme()
{
    destroy();
}

void FXTheme::reset( FXuint cls )
{
    if (cls < 0 || cls >= THEME_LAST)
        fxerror("%s: invalid theme index %d\n", m_window->getClassName(), cls);

    if (!m_window->id())
        return;

    // If themes were marked as inactive or if this theme already has
    // the given class, there is nothing else to do.
    if (m_class == THEME_INACTIVE || m_class == cls)
        return;

    // The widget doesn't care about the class
    if (cls == THEME_ANY && m_class != THEME_NONE)
        return;

    destroy();

#ifdef HKS_NT
    if (!bThemeLibLoaded || !zIsThemeActive()) {
        m_class = THEME_INACTIVE;
        return;
    }

    // WARNING: This *must* be in synch with FXThemeClass
    static wchar_t *wClsName[] =
    {
        L"",                // THEME_NONE
        L"WINDOW",          // THEME_ANY
        L"BUTTON",          // THEME_BUTTON
        L"COMBOBOX",        // THEME_COMBOBOX
        L"BUTTON",          // THEME_GROUPBOX
        L"EDIT",            // THEME_EDIT
        L"HEADER",          // THEME_HEADER
        L"PROGRESS",        // THEME_PROGRESS
        L"SCROLLBAR",       // THEME_SCROLLBAR
        L"TRACKBAR",        // THEME_SLIDER
        L"SPIN",            // THEME_SPINNER
        L"TAB",             // THEME_TAB
        L"TOOLBAR",         // THEME_TOOLBAR
        L"TREEVIEW",        // THEME_TREE
        L"WINDOW"           // THEME_TOPWINDOW
    };

    m_theme = (FXID) zOpenThemeData( (HWND) m_window->id(), wClsName[cls]);
#else
    m_class = THEME_INACTIVE;
#endif

    if (m_theme)
        m_class = cls;
}

void FXTheme::destroy()
{
#ifdef HKS_NT
    if (m_theme)
        zCloseThemeData( (HTHEME) m_theme );
#endif
    m_theme = NULL;
    m_class = THEME_NONE;
}

void FXTheme::initialize()
{
#ifdef HKS_NT
    InitThemes();
#endif
}

void FXTheme::finalize()
{
#ifdef HKS_NT
    FinThemes();
#endif
}

FXSize FXTheme::getDefaultSize( int partId, int stateId )
{
#ifdef HKS_NT
    if (m_theme) {
        SIZE sz;
        HDC hdc = GetDC(NULL);
        HRESULT res = zGetThemePartSize((HTHEME) m_theme, hdc, partId, stateId, NULL, TS_TRUE, &sz);
        ReleaseDC(NULL,hdc);
        if (res == S_OK)
            return FXSize(sz.cx, sz.cy);
    }
#endif
    return FXSize(1,1);
}


#ifdef HKS_NT
// Need to resort to low level tricks in order to extract the DC handle.
// Starting FOX 1.2 there is a function in FXDCWindow that will return 
// the context handle.
struct FXDCHack : public FXDCWindow {
    FXID context() const { return dc; }
};
#endif

void FXTheme::drawBackground(
    FXDCWindow &dc, FXint partId, FXint stateId,
    FXint x, FXint y, FXint w, FXint h,
    FXint clipX, FXint clipY, FXint clipW, FXint clipH,
    Orientation orientation
    )
{
#ifdef HKS_NT
    if (m_theme) {
        HDC hdc = (HDC) static_cast<FXDCHack&>(dc).context();
        int old = SetBkMode( hdc, TRANSPARENT );

        // If no rectangle given, draw the whole rectangle
        if (x == 0 && y == 0 && w == 0 && h == 0) {
            w = m_window->getWidth();
            h = m_window->getHeight();
        }
        RECT rc = { x, y, x+w, y+h };

        // If no clip rectangle given, draw everything
        RECT clipRc = { clipX, clipY, clipX+clipW, clipY+clipH };
        RECT *pClipRc = &clipRc;
        if (clipX == 0 && clipY == 0 && clipW == 0 && clipH == 0)
            pClipRc = NULL;

        if (orientation == UP)
            zDrawThemeBackground( (HTHEME) m_theme, hdc, partId, stateId, &rc, pClipRc );
        else {
            FXImage img(FXApp::instance());

            if (orientation == LEFT || orientation == RIGHT) {
                img.resize(h,w);
                img.create();
                FXDCWindow buffDc(&img);
                HDC hdc = (HDC) static_cast<FXDCHack&>(buffDc).context();
                RECT rc = { 0, 0, h, w };
                zDrawThemeBackground( (HTHEME) m_theme, hdc, partId, stateId, &rc, NULL );
                img.restore();
                img.rotate(orientation == LEFT ? 90 : 270);
            }
            else if (orientation == DOWN) {
                img.resize(w,h);
                img.create();
                FXDCWindow buffDc(&img);
                HDC hdc = (HDC) static_cast<FXDCHack&>(buffDc).context();
                RECT rc = { 0, 0, w, h };
                zDrawThemeBackground( (HTHEME) m_theme, hdc, partId, stateId, &rc, NULL );
                img.restore();
                img.rotate(180);
            }

            if (pClipRc)
                dc.drawArea( &img, clipX, clipY, clipW, clipH, clipX - x, clipY - y );
            else
                dc.drawArea( &img, x, y, w, h, 0, 0 );
        }

        SetBkMode( hdc, old );
    }
#endif
}


void FXTheme::drawEdge(
    FXDCWindow &dc, int partId, int stateId, FXuint frameStyle,
    int x, int y, int w, int h )
{
#ifdef HKS_NT
    if (m_theme) {
        HDC hdc = (HDC) static_cast<FXDCHack&>(dc).context();
        int old = SetBkMode( hdc, TRANSPARENT );

        int edge=0, flags=0;
        switch (frameStyle) {
        case FRAME_LINE: edge = BDR_SUNKENOUTER; flags = BF_RECT|BF_MONO; break;
        case FRAME_SUNKEN: edge = BDR_SUNKENINNER; flags = BF_RECT; break;
        case FRAME_RAISED: edge = BDR_RAISEDINNER; flags = BF_RECT; break;
        case FRAME_GROOVE: edge = EDGE_ETCHED; flags = BF_RECT; break;
        case FRAME_RIDGE:  edge = EDGE_BUMP; flags = BF_RECT; break;
        case FRAME_SUNKEN|FRAME_THICK: edge = EDGE_SUNKEN; flags = BF_RECT; break;
        case FRAME_RAISED|FRAME_THICK: edge = EDGE_RAISED; flags = BF_RECT; break;
        }

        if (edge || flags) {
            // If no rectangle given, draw the whole rectangle
            if (x == 0 && y == 0 && w == 0 && h == 0) {
                w = m_window->getWidth();
                h = m_window->getHeight();
            }
            RECT rc = { x, y, x+w, y+h };
            zDrawThemeEdge( (HTHEME) m_theme, hdc, partId, stateId, &rc, edge, flags, NULL );
        }
        SetBkMode( hdc, old );
    }
#endif
}

void FXTheme::setWindowRegion(
    FXDCWindow &dc, int partId, int stateId,
    int x, int y, int w, int h
    )
{
#ifdef HKS_NT
    if (m_theme) {
        HDC hdc = (HDC) static_cast<FXDCHack&>(dc).context();
        // If no rectangle given, draw the whole rectangle
        if (x == 0 && y == 0 && w == 0 && h == 0) {
            w = m_window->getWidth();
            h = m_window->getHeight();
        }
        RECT rc = { x, y, x+w, y+h };
        HRGN hrgn;
        if (zGetThemeBackgroundRegion((HTHEME) m_theme, hdc, partId, stateId, &rc, &hrgn) == S_OK)
            SetWindowRgn( (HWND) m_window->id(), hrgn, TRUE );
    }
#endif
}

void FXTheme::clearWindowRegion()
{
#ifdef HKS_NT
    SetWindowRgn( (HWND) m_window->id(), NULL, FALSE );
#endif
}


void FXTheme::excludePartRegion(
    FXDCWindow &dc, int partId, int stateId,
    int x, int y, int w, int h
    )
{
#ifdef HKS_NT
    if (m_theme) {
        HDC hdc = (HDC) static_cast<FXDCHack&>(dc).context();
        // If no rectangle given, draw the whole rectangle
        if (x == 0 && y == 0 && w == 0 && h == 0) {
            w = m_window->getWidth();
            h = m_window->getHeight();
        }
        RECT rc = { x, y, x+w, y+h };
        HRGN hrgn;
        if (zGetThemeBackgroundRegion((HTHEME) m_theme, hdc, partId, stateId, &rc, &hrgn) == S_OK) {
            ExtSelectClipRgn( hdc, hrgn, RGN_DIFF );
            DeleteObject( hrgn );
        }
    }
#endif
}

void FXTheme::offsetOrigin( FXDCWindow &dc, int dx, int dy )
{
#ifdef HKS_NT
    if (m_theme) {
        HDC hdc = (HDC) static_cast<FXDCHack&>(dc).context();
        OffsetViewportOrgEx(hdc, dx, dy, NULL);
    }
#endif
}

FXColor FXTheme::getThemeTextColor( int partId, int stateId )
{
#ifdef HKS_NT
    if (m_theme) {
        COLORREF clr = 0;
        zGetThemeColor( (HTHEME) m_theme, partId, stateId, TMT_TEXTCOLOR, &clr );
        return FXRGB(GetRValue(clr), GetGValue(clr), GetBValue(clr));
    }
#endif
    return 0;
}

FXColor FXTheme::getSysMenuColor()
{
#ifdef HKS_NT
    if (m_theme) {
        COLORREF clr = zGetThemeSysColor((HTHEME)m_theme, COLOR_MENU);
        return FXRGB(GetRValue(clr), GetGValue(clr), GetBValue(clr));
    }
#endif
    return 0;
}

FXColor FXTheme::getSysMenuHighlight()
{
#ifdef HKS_NT
    if (m_theme) {
        COLORREF clr = zGetThemeSysColor((HTHEME)m_theme, COLOR_HIGHLIGHT);
        return FXRGB(GetRValue(clr), GetGValue(clr), GetBValue(clr));
    }
#endif
    return 0;
}

FXColor FXTheme::getSysCaptionTextColor()
{
#ifdef HKS_NT
    if (m_theme) {
        COLORREF clr = zGetThemeSysColor((HTHEME)m_theme, COLOR_CAPTIONTEXT);
        return FXRGB(GetRValue(clr), GetGValue(clr), GetBValue(clr));
    }
#endif
    return 0;
}

FXColor FXTheme::getSysInactiveCaptionTextColor()
{
#ifdef HKS_NT
    if (m_theme) {
        COLORREF clr = zGetThemeSysColor((HTHEME)m_theme, COLOR_INACTIVECAPTIONTEXT);
        return FXRGB(GetRValue(clr), GetGValue(clr), GetBValue(clr));
    }
#endif
    return 0;
}

FXColor FXTheme::getSysAppWorkspaceColor()
{
#ifdef HKS_NT
    if (m_theme) {
        COLORREF clr = zGetThemeSysColor((HTHEME)m_theme, COLOR_APPWORKSPACE);
        return FXRGB(GetRValue(clr), GetGValue(clr), GetBValue(clr));
    }
#endif
    return 0;
}

FXint FXTheme::getSysVerticalScrollbarWidth()
{
#ifdef HKS_NT
    if (m_theme)
        return zGetThemeSysSize((HTHEME)m_theme, SM_CXVSCROLL);
#endif
    return 0;
}

FXint FXTheme::getSysHorizontalScrollbarHeight()
{
#ifdef HKS_NT
    if (m_theme)
        return zGetThemeSysSize((HTHEME)m_theme, SM_CYHSCROLL);
#endif
    return 0;
}
