REM Platform: Windows/x86-32


REM Define the top directory where the FOX source code and build scripts are
REM located.  This is the directory that was created when you unzipped the
REM package downloaded from the ABAQUS FTP site.
REM Note: The default setting for CAE_FOX_DIR is ..\.. because the build script
REM       is located in a directory two levels down from the top FOX directory.
REM       It would work fine if you run the script in that directory.  If you
REM       copy the script and the related files to a different directory and
REM       run the script there, redefine CAE_FOX_DIR accordingly.  Do not use
REM       spaces after = in the set command.
REM
set CAE_FOX_DIR=..\..


REM Define whether the support for 3Dconnexion motion controllers (e.g.,
REM SpaceBall) is desired.  In order for ABAQUS/CAE to support those motion
REM controllers using your version of FOX libraries, you need to download the
REM Software Developer's Kits (SDK) from 3Dconnexion and build the kit before
REM running this script.  Please refer to the README file for more information
REM about this SDK.  If you do not need this support, set HAVE_SPACEWARE to 0.
REM Note: SPACEWARE_INC_DIR is the directroy where the SDK header files are
REM       located.  By default those header files are installed in the
REM       following directory after you download the kit:
REM           C:\Program Files\3Dconnexion\3DxWare SDK\inc
REM       SPACEWARE_OBJ_DIR is the directroy where you store the SDK object
REM       file after you build the kit.  You must fill in this directroy path
REM       in order for the script to build FOX libraries successfully if you
REM       set HAVE_SPACEWARE to 1.  Do not use spaces after = in any of the set
REM       commands.
REM
set HAVE_SPACEWARE=1
set SPACEWARE_INC_DIR="C:\Program Files\3Dconnexion\3DxWare SDK\inc"
set SPACEWARE_OBJ_DIR=


REM *****
REM ***** Please do not make any changes beyond this point in the script. *****
REM *****


REM Build shared library ABQFox.dll from all FOX source files that do not
REM require OpenGL.
REM
if %HAVE_SPACEWARE% == 1 (
    set SPW_DEF=/DHAVE_SPW /DOS_WIN32
    set SPW_INC=/I%SPACEWARE_INC_DIR%
    set SPW_OBJ=%SPACEWARE_OBJ_DIR%\*.obj
) else (
    set SPW_DEF=
    set SPW_INC=
    set SPW_OBJ=
)
LIB /OUT:spwObjects.lib spwObjects\*.obj %SPW_OBJ%

set COMPILE=CL /c /TP /nologo /Zl /GX /vmg /vmm /DWIN32 /D_WINDOWS /DHKS_NT /DIMPDLL /DNTI /DNT /DNOUSER /DNOMINMAX /DHKS_OPEN_GL %SPW_DEF% /DFOXDLL -DNDEBUG -DTYPENAME= -DHKS_DEBUG /DFOXDLL_EXPORTS /W1 /DWARNING_IGNORE="4355 4786" /DWARNING_ERROR="4530 4706" /Og /Oi /Ot /Ob1 /Gs /Gy /Z7 /MD

set INC_OPTIONS=/I%CAE_FOX_DIR%\include /I%CAE_FOX_DIR%\src /I%CAE_FOX_DIR%\abaqus %SPW_INC%

%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FX4Splitter.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXAccelTable.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXApp.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXArrowButton.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXBitmap.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXBMPIcon.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXBMPImage.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\fxbmpio.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXButton.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXCanvas.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXCharset.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXCheckButton.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXColorBar.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXColorDialog.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXColorNames.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXColorSelector.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXColorWell.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXColorWheel.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXComboBox.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXComposite.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXCURCursor.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\fxcurio.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXCursor.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDataTarget.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDC.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDCPrint.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDCWindow.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDebugTarget.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDelegator.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDHMat.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDHVec.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDial.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDialogBox.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDict.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDirBox.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDirDialog.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDirList.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDirSelector.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDocument.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDQuat.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDragCorner.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDrawable.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDriveBox.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXDVec.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXException.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXFile.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXFileDialog.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXFileDict.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXFileList.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\fxfilematch.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXFileSelector.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXFont.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXFontDialog.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXFontSelector.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXFrame.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXGIFCursor.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXGIFIcon.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXGIFImage.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\fxgifio.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXGroupBox.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXHeader.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXHMat.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXHorizontalFrame.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXHVec.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXICOIcon.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXICOImage.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\fxicoio.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXIcon.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXIconList.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXId.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXImage.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXImageView.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXInputDialog.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\fxjpegio.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXJPGIcon.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXJPGImage.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXLabel.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXList.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXListBox.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXMainWindow.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXMatrix.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXMDIButton.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXMDIChild.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXMDIClient.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXMenubar.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXMenuButton.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXMenuCaption.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXMenuCascade.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXMenuCommand.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXMenuPane.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXMenuSeparator.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXMenuTitle.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXMessageBox.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXObject.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXObjectList.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXOptionMenu.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXPacker.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXPCXIcon.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXPCXImage.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\fxpcxio.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXPicker.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXPNGIcon.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXPNGImage.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\fxpngio.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXPoint.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXPopup.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXPrintDialog.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\fxpriv.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXProgressBar.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXProgressDialog.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\fxquantize.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXQuat.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXRadioButton.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXRange.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXRecentFiles.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXRectangle.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXRegion.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXRegistry.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXReplaceDialog.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXRex.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXRGBIcon.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXRGBImage.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\fxrgbio.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXRootWindow.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXScrollArea.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXScrollbar.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXScrollWindow.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXSearchDialog.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXSeparator.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXSettings.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXShell.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXShutter.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXSize.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXSlider.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXSpinner.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXSplitter.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXStatusbar.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXStatusline.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXStream.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXString.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXStringDict.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXSwitcher.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXTabBar.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXTabBook.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXTabItem.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXTable.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\fxtargaio.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXText.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXTextField.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXTGAIcon.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXTGAImage.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXTIFIcon.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXTIFImage.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\fxtifio.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXToggleButton.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXToolbar.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXToolbarGrip.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXToolbarShell.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXToolbarTab.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXTooltip.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXTopWindow.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXTreeList.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXTreeListBox.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXUndoList.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXURL.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\fxutils.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXVec.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXVerticalFrame.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXVisual.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXWindow.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\fxwinkbd.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXXPMIcon.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXXPMImage.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\fxxpmio.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\vsscanf.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\abaqus\FXTheme.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\abaqus\Foxfox_initialize.C

LINK @foxLinkOpts.txt @foxObjects.lis @foxMoreLibs.lis


REM Build shared library ABQfgl.so from all FOX source files that require
REM OpenGL.
REM
set COMPILE=CL /c /TP /nologo /Zl /GX /vmg /vmm /DWIN32 /D_WINDOWS /DHKS_NT /DIMPDLL /DNTI /DNT /DNOUSER /DNOMINMAX /DHKS_OPEN_GL /DFOXDLL -DNDEBUG -DTYPENAME= -DHKS_DEBUG /W1 /DWARNING_IGNORE="4355 4786" /DWARNING_ERROR="4530 4706" /Og /Oi /Ot /Ob1 /Gs /Gy /Z7 /MD

set INC_OPTIONS=/I%CAE_FOX_DIR%\include /I%CAE_FOX_DIR%\src /I%CAE_FOX_DIR%\abaqus

%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXGLCanvas.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXGLCone.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXGLContext.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXGLCube.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXGLCylinder.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXGLObject.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXGLShape.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXGLSphere.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXGLTriangleMesh.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXGLViewer.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\src\FXGLVisual.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\abaqus\FXGLContextEx.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\abaqus\FXGLDC.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\abaqus\FXGLImage.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\abaqus\FXGLVisualEx.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\abaqus\FXOVVisual.C
%COMPILE% %INC_OPTIONS% %CAE_FOX_DIR%\abaqus\Foxfgl_initialize.C

LINK @fglLinkOpts.txt @fglObjects.lis @fglMoreLibs.lis


REM Clean up.
REM
del *.obj
