/********************************************************************************
*                                                                               *
*                           S t r i n g   O b j e c t                           *
*                                                                               *
*********************************************************************************
* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
*********************************************************************************
* $Id: FXString.h,v 1.38 2002/02/08 14:27:20 fox Exp $                          *
*********************************************************************************
*  CAE MODIFICATIONS:                                                           *
*  This class has been modified by CAE to derive from a common base class which * 
*  is used by cow_String and atr_TRString - a translated string class used to   *
*  ensure corect i18n trnaslations.  By using a common base class it should be  *
*  possible to covert between strings through base class construction.          *
*  It will not be possible to directly create a translated string from the other*
*  two string classes.	                                                        *
*  For methods in the base class, FXString will just call the base class method.*
*  For the remaining string massging methods, FXString will use the base class  *
*  management methods.							        *
********************************************************************************/
#ifndef FXString_H
#define FXString_H
// begin local includes
#ifndef XINCS_H
#include <xincs.h>
#endif
#ifndef FXDEFS_H
#include <fxdefs.h>
#endif

//CAE -- add base class
#include <atr_StringBase.h>

// CAE: 
class FXString;
FXAPI FXString FXStringFormat(const FXchar *fmt,...);
FXAPI FXString FXStringVal(FXint num,FXint base=10);
FXAPI FXString FXStringVal(FXuint num,FXint base=10);
FXAPI FXString FXStringVal(FXfloat num,FXint prec=6,FXbool exp=MAYBE);
FXAPI FXString FXStringVal(FXdouble num,FXint prec=6,FXbool exp=MAYBE);
FXAPI FXint compare(const FXchar *s1,const FXchar *s2);
FXAPI FXint compare(const FXchar *s1,const FXString &s2);
FXAPI FXint compare(const FXString &s1,const FXchar *s2);
FXAPI FXint compare(const FXString &s1,const FXString &s2);
FXAPI FXint compare(const FXchar *s1,const FXchar *s2,FXint n);
FXAPI FXint compare(const FXchar *s1,const FXString &s2,FXint n);
FXAPI FXint compare(const FXString &s1,const FXchar *s2,FXint n);
FXAPI FXint compare(const FXString &s1,const FXString &s2,FXint n);
FXAPI FXint comparecase(const FXchar *s1,const FXchar *s2);
FXAPI FXint comparecase(const FXchar *s1,const FXString &s2);
FXAPI FXint comparecase(const FXString &s1,const FXchar *s2);
FXAPI FXint comparecase(const FXString &s1,const FXString &s2);
FXAPI FXint comparecase(const FXchar *s1,const FXchar *s2,FXint n);
FXAPI FXint comparecase(const FXchar *s1,const FXString &s2,FXint n);
FXAPI FXint comparecase(const FXString &s1,const FXchar *s2,FXint n);
FXAPI FXint comparecase(const FXString &s1,const FXString &s2,FXint n);
FXAPI FXString FXStringFormat(const FXchar *fmt,...) FX_PRINTF(1,2) ;
FXAPI FXString FXStringVFormat(const FXchar *fmt,va_list args);


/**
* FXString provides essential string manipulation capabilities.
*/
class FXAPI FXString 
    :public atr_StringBase  //CAE add base class
{
private:
public:
  static const FXchar null[];
  static const FXchar hex[17];
  static const FXchar HEX[17];
public:

  /// Create empty string
  FXString()
  { }  // CAE inline ctor for performance

  /// Copy construct
  FXString(const FXString& s)
	: atr_StringBase(s)  //CAE inline copy ctor
    { }

  /// Construct and init
  FXString(const FXchar* s)
      :atr_StringBase(s?s:"") // CAE inline char* CTOR
    { }


  /// Construct and init with substring
  FXString(const FXchar* s,FXint n)
    :atr_StringBase (s?s:"",n)  //CAE inline substring ctor
    {}

  /// Construct and fill with constant
  FXString(FXchar c,FXint n);

  /// Construct string from two parts
  FXString(const FXchar *s1,const FXchar* s2);

  /// Constructor from base class
  FXString (const atr_StringBase& rhs)
	: atr_StringBase(rhs)  //CAE new base class ctor
    { }

  /// Size to some desired capacity
  void size(FXint sz);

  /// Size of text data, 0 for empty string
  FXint size() const;

  /// Length of text
  FXint length() const { return strlen(CStr()); }  //CAE use base class implementation

  /// Get text contents
  const FXchar* text() const { return CStr(); }//CAE use base class implementation

  /// See if string is empty
  FXbool empty() const { return m_StringRep == 0 ? true:CStr()[0] == ATR_EOS; }//CAE use base class implementation

  /// Return a non-const reference to the ith character
  FXchar& operator[](FXint i);//CAE different than base class since can get last eos char

  /// Return a const reference to the ith character
  const FXchar& operator[](FXint i) const{return text()[i];}//CAE use base class implementation

  /// Assign another string to this
  inline FXString& operator=(const FXString& s);  //CAE Use base class and inline

  /// Assign a C-style string to this
  inline FXString& operator=(const FXchar* s);	//CAE Use base class and inline

  /// Assign a base class string to this
  inline FXString& operator = (const atr_StringBase& str);  //CAE New assignment of base class

  /// Fill with a constant
  FXString& fill(FXchar c,FXint n);

  /// Fill up to current length
  FXString& fill(FXchar c);

  /// Convert to lower case
  inline FXString& lower();//CAE Use base class and inline

  /// Convert to upper case
  inline FXString& upper();//CAE Use base class and inline

  /// Extract partition of delimiter separated string
  FXString extract(FXint part,FXchar delim) const;

  /// Extract partition of delimiter separated string
  FXString extract(FXint part,FXchar delim,FXchar esc) const;

  /// Insert character at specified position
  FXString& insert(FXint pos,FXchar c);

  /// Insert n characters c at specified position
  FXString& insert(FXint pos,FXchar c,FXint n);

  /// Insert first n characters of string at specified position
  FXString& insert(FXint pos,const FXchar* s,FXint n);

  /// Insert string at specified position
  FXString& insert(FXint pos,const FXString& s);

  /// Insert string at specified position
  FXString& insert(FXint pos,const FXchar* s);

  /// Prepend string with input character
  inline FXString& prepend(FXchar c);//CAE inline for performance

  /// Prepend string with n characters c
  inline FXString& prepend(FXchar c,FXint n);//CAE inline for performance

  /// Prepend string with first n characters of input string
  FXString& prepend(const FXchar *s,FXint n);

  /// Prepend string with input string
  inline FXString& prepend(const FXString& s);//CAE inline for performance

  /// Prepend string with input string
  FXString& prepend(const FXchar *s);

  /// Append input character to this string
  inline FXString& append(FXchar c);//CAE inline for performance

  /// Append input n characters c to this string
  FXString& append(FXchar c,FXint n);

  /// Append first n characters of input string to this string
  FXString& append(const FXchar *s,FXint n);

  /// Append input string to this string
  FXString& append(const FXString& s);

  /// Append input string to this string
  FXString& append(const FXchar *s);

  /// Replace a single character
  FXString& replace(FXint pos,FXchar c);

  /// Replace the m characters at pos with n characters c
  FXString& replace(FXint pos,FXint m,FXchar c,FXint n);

  /// Replaces the m characters at pos with first n characters of input string
  FXString& replace(FXint pos,FXint m,const FXchar *s,FXint n);

  /// Replace the m characters at pos with input string
  inline FXString& replace(FXint pos,FXint m,const FXString& s);//CAE inline for performance

  /// Replace the m characters at pos with input string
  FXString& replace(FXint pos,FXint m,const FXchar *s);

  /// Remove substring
  FXString& remove(FXint pos,FXint n=1);

  /// Substitute one character by another
  FXString& substitute(FXchar orig,FXchar sub);

  /// Remove leading and trailing whitespace
  FXString& trim();

  /// Remove leading whitespace
  FXString& trimBegin();

  /// Remove trailing whitespace
  FXString& trimEnd();

  /// Truncate string at pos
  FXString& trunc(FXint pos);

  /// Clear
  inline FXString& clear();  //CAE Use base class and inline

  /// Get leftmost part
  FXString left(FXint n) const;

  /// Get rightmost part
  FXString right(FXint n) const;

  /// Get some part in the middle
  FXString mid(FXint pos,FXint n) const;

  /**
  * Return all characters before the n-th occurrence of ch,
  * counting from the beginning of the string if n>0, or
  * from the end if n<0.
  * If the character ch is not found, the entire string,
  * or the empty string, is returned, respectively.
  * A NULL string is returned if n==0.
  */
  FXString before(FXchar ch,FXint n=1) const;

  /**
  * Return all characters after the nth occurrence of ch,
  * counting from the beginning of the string if n>0, or from
  * the end if n<0.
  * If the character ch is not found, the empty string, or
  * the entire string, is returned, respectively.
  * A NULL string is returned if n==0.
  */
  FXString after(FXchar ch,FXint n=1) const;

  /// Find a character, searching forward; return position or -1
  FXint findf(FXchar c,FXint pos=0) const;

  /// Find a character, searching backward; return position or -1
  FXint findb(FXchar c,FXint pos=2147483647) const;

  /// Find a substring of length n, searching forward; return position or -1
  FXint findf(const FXchar* substr,FXint n,FXint pos) const;

  /// Find a substring of length n, searching backward; return position or -1
  FXint findb(const FXchar* substr,FXint n,FXint pos) const;

  /// Find a substring, searching forward; return position or -1
  FXint findf(const FXchar* substr,FXint pos=0) const;

  /// Find a substring, searching backward; return position or -1
  FXint findb(const FXchar* substr,FXint pos=2147483647) const;

  /// Find a substring, searching forward; return position or -1
  FXint findf(const FXString &substr,FXint pos=0) const;

  /// Find a substring, searching backward; return position or -1
  FXint findb(const FXString &substr,FXint pos=2147483647) const;

  /// Find number of occurances of character in string
  FXint count(FXchar c) const;

  /// Format a string a-la printf
  FXString& format(const char *fmt,...) FX_PRINTF(2,3) ;
  FXString& vformat(const char *fmt,va_list args);

  /// Scan a string a-la scanf
  FXint scan(const char *fmt,...) const FX_SCANF(2,3) ;
  FXint vscan(const char *fmt,va_list args) const;

  /// Get hash value
  FXint hash() const;

  /// Compare
  friend FXAPI FXint compare(const FXchar *s1,const FXchar *s2);
  friend FXAPI FXint compare(const FXchar *s1,const FXString &s2);
  friend FXAPI FXint compare(const FXString &s1,const FXchar *s2);
  friend FXAPI FXint compare(const FXString &s1,const FXString &s2);

  /// Compare up to n
  friend FXAPI FXint compare(const FXchar *s1,const FXchar *s2,FXint n);
  friend FXAPI FXint compare(const FXchar *s1,const FXString &s2,FXint n);
  friend FXAPI FXint compare(const FXString &s1,const FXchar *s2,FXint n);
  friend FXAPI FXint compare(const FXString &s1,const FXString &s2,FXint n);

  /// Compare case insensitive
  friend FXAPI FXint comparecase(const FXchar *s1,const FXchar *s2);
  friend FXAPI FXint comparecase(const FXchar *s1,const FXString &s2);
  friend FXAPI FXint comparecase(const FXString &s1,const FXchar *s2);
  friend FXAPI FXint comparecase(const FXString &s1,const FXString &s2);

  /// Compare case insensitive up to n
  friend FXAPI FXint comparecase(const FXchar *s1,const FXchar *s2,FXint n);
  friend FXAPI FXint comparecase(const FXchar *s1,const FXString &s2,FXint n);
  friend FXAPI FXint comparecase(const FXString &s1,const FXchar *s2,FXint n);
  friend FXAPI FXint comparecase(const FXString &s1,const FXString &s2,FXint n);

  /// Comparison operators
  friend FXAPI FXbool operator==(const FXString &s1,const FXString &s2);
  friend FXAPI FXbool operator==(const FXString &s1,const FXchar *s2);
  friend FXAPI FXbool operator==(const FXchar *s1,const FXString &s2);

  friend FXAPI FXbool operator!=(const FXString &s1,const FXString &s2);
  friend FXAPI FXbool operator!=(const FXString &s1,const FXchar *s2);
  friend FXAPI FXbool operator!=(const FXchar *s1,const FXString &s2);

  friend FXAPI FXbool operator<(const FXString &s1,const FXString &s2);
  friend FXAPI FXbool operator<(const FXString &s1,const FXchar *s2);
  friend FXAPI FXbool operator<(const FXchar *s1,const FXString &s2);

  friend FXAPI FXbool operator<=(const FXString &s1,const FXString &s2);
  friend FXAPI FXbool operator<=(const FXString &s1,const FXchar *s2);
  friend FXAPI FXbool operator<=(const FXchar *s1,const FXString &s2);

  friend FXAPI FXbool operator>(const FXString &s1,const FXString &s2);
  friend FXAPI FXbool operator>(const FXString &s1,const FXchar *s2);
  friend FXAPI FXbool operator>(const FXchar *s1,const FXString &s2);

  friend FXAPI FXbool operator>=(const FXString &s1,const FXString &s2);
  friend FXAPI FXbool operator>=(const FXString &s1,const FXchar *s2);
  friend FXAPI FXbool operator>=(const FXchar *s1,const FXString &s2);

  /// Append operators
  inline FXString& operator+=(const FXString& s);//CAE inline for performance
  FXString& operator+=(const FXchar* s);
  inline FXString& operator+=(FXchar c);//CAE inline for performance

  /// Concatenate two strings
  friend FXAPI FXString operator+(const FXString& s1,const FXString& s2);
  friend FXAPI FXString operator+(const FXString& s1,const FXchar* s2);
  friend FXAPI FXString operator+(const FXchar* s1,const FXString& s2);

  /// Concatenate with single character
  friend FXAPI FXString operator+(const FXString& s,FXchar c);
  friend FXAPI FXString operator+(FXchar c,const FXString& s);

  /// Saving to a stream
  friend FXAPI FXStream& operator<<(FXStream& store,const FXString& s);

  /// Load from a stream
  friend FXAPI FXStream& operator>>(FXStream& store,FXString& s);

  /// Format a string a-la printf
  friend FXAPI FXString FXStringFormat(const FXchar *fmt,...) FX_PRINTF(1,2) ;
  friend FXAPI FXString FXStringVFormat(const FXchar *fmt,va_list args);

  /**
  * Convert integer number to a string, using the given number
  * base, which must be between 2 and 16.
  */
  // CAE:
  // friend FXAPI FXString FXStringVal(FXint num,FXint base=10);
  // friend FXAPI FXString FXStringVal(FXuint num,FXint base=10);
  friend FXAPI FXString FXStringVal(FXint num,FXint base);
  friend FXAPI FXString FXStringVal(FXuint num,FXint base);

  /**
  * Convert real number to a string, using the given procision and
  * exponential notation mode, which may be FALSE (never), TRUE (always), or
  * MAYBE (when needed).
  */
  // CAE:
  // friend FXAPI FXString FXStringVal(FXfloat num,FXint prec=6,FXbool exp=MAYBE);
  // friend FXAPI FXString FXStringVal(FXdouble num,FXint prec=6,FXbool exp=MAYBE);
  friend FXAPI FXString FXStringVal(FXfloat num,FXint prec,FXbool exp);
  friend FXAPI FXString FXStringVal(FXdouble num,FXint prec,FXbool exp);

  /// Convert string to a integer number, assuming given number base
  friend FXAPI FXint FXIntVal(const FXString& s,FXint base=10);
  friend FXAPI FXuint FXUIntVal(const FXString& s,FXint base=10);

  /// Convert string into real number
  friend FXAPI FXfloat FXFloatVal(const FXString& s);
  friend FXAPI FXdouble FXDoubleVal(const FXString& s);

  };

//CAE Use base class and inline for assignment
inline FXString& FXString::operator = (const FXString& rhs)
{
    atr_StringBase::operator = (rhs);
    return *this; 
}

//CAE Use base class and inline for assignment
inline FXString& FXString::operator = (const FXchar* rhs)
{
    atr_StringBase::operator = (rhs?rhs:"");   
    return *this; 
}

inline FXString& FXString::operator = (const atr_StringBase& rhs)
{
    atr_StringBase::operator = (rhs);
    return *this; 
}
// CAE Use base class and inline to change the case
inline FXString&   FXString::lower()
{
    atr_StringBase::ToLower();
    return *this; 
}

// CAE Use base class and inline to change the case
inline FXString&   FXString::upper()
{
    atr_StringBase::ToUpper();
    return *this; 
}

// Prepend character
inline FXString& FXString::prepend(FXchar c){
  return insert(0,c);
  }

inline FXString& FXString::prepend(FXchar c,FXint n){
  return insert(0,c,n);
  }

// Prepend string
inline FXString& FXString::prepend(const FXString& s){
  return insert(0,s);
  }


// Add character to the end
inline FXString& FXString::append(FXchar c){
  atr_StringBase::Append (&c,1);
  return *this;
  }

// Append FXString
inline FXString& FXString::operator+=(const FXString& s){
  append(s.CStr(),s.Length());
  return *this;
  }

// Append character
inline FXString& FXString::operator+=(FXchar c){
  append(c);
  return *this;
  }



// Replace part of string
inline FXString& FXString::replace(FXint pos,FXint m,const FXString& s){
  replace(pos,m,s.CStr(),s.Length());
  return *this;
  }

// Clean string
inline FXString& FXString::clear(){
  atr_StringBase::Erase(0, atr_StrEnd);
  return *this;
  }


#endif
