// begin local includes
/********************************************************************************
*                                                                               *
*                G r o u p  B o x   W i n d o w   O b j e c t                   *
*                                                                               *
*********************************************************************************
* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
*********************************************************************************
* $Id: FXGroupBox.cpp,v 1.20 2002/01/18 22:43:00 jeroen Exp $                   *
********************************************************************************/
#include <xincs.h>
#include <fxver.h>
#include <fxdefs.h>
#include <FXStream.h>
#include <FXString.h>
#include <FXSize.h>
#include <FXPoint.h>
#include <FXRectangle.h>
#include <FXSettings.h>
#include <FXRegistry.h>
#include <FXApp.h>
#include <FXDC.h>
#include <FXDCWindow.h>
#include <FXFont.h>
#include <FXLabel.h>
#include <FXGroupBox.h>

#include <FXTheme.h>    // CAE
#include <fxtmschema.h> // CAE

#define CAE_I18N

#ifdef CAE_I18N
// This will be called by AFXApp upon initialization.
static FXFont *gbFont;
void hks_setFXGroupBoxFont(FXFont *font) { gbFont = font; }
#endif

/*
  Notes:

  - Having FXGroupBox enforce radio behaviour isn't as useful anymore when
    you already have GUI Update mechanism.  Perhaps this should be dropped.
*/


#define FRAME_MASK           (FRAME_SUNKEN|FRAME_RAISED|FRAME_THICK)
#define GROUPBOX_TITLE_MASK  (GROUPBOX_TITLE_LEFT|GROUPBOX_TITLE_CENTER|GROUPBOX_TITLE_RIGHT)


/*******************************************************************************/

// Map
FXDEFMAP(FXGroupBox) FXGroupBoxMap[]={
  FXMAPFUNC(SEL_PAINT,0,FXGroupBox::onPaint),
  FXMAPFUNC(SEL_UNCHECK_OTHER,0,FXGroupBox::onUncheckOther),
  };


// Object implementation
FXIMPLEMENT(FXGroupBox,FXPacker,FXGroupBoxMap,ARRAYNUMBER(FXGroupBoxMap))


// Deserialization
FXGroupBox::FXGroupBox(){
  flags|=FLAG_ENABLED;
  font=(FXFont*)-1;
  textColor=0;
  }


// Make a groupbox
  // CAE use translated string for displayed text
FXGroupBox::FXGroupBox(FXComposite* p,const atr_String& text,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs):
  FXPacker(p,opts,x,y,w,h,pl,pr,pt,pb,hs,vs),label(text){
  flags|=FLAG_ENABLED;
#ifdef CAE_I18N
  font = gbFont;
#else
  font=getApp()->getNormalFont();
  // Begin HKS CAE additions
  FXString fName = font->getName();
  //  FXuint fSize = font->getSize() - 20;  // Reduced size may look better
  FXuint fSize = font->getSize();
  font = new FXFont(getApp(),fName,fSize/10,FONTWEIGHT_BOLD);
  // End HKS CAE additions
#endif
  textColor=getApp()->getForeColor();
  }


// Create window
void FXGroupBox::create(){
  FXPacker::create();
#ifdef CAE_I18N
  theme.reset(THEME_GROUPBOX);
  if (!font || theme.active())
      font=getApp()->getNormalFont();
#endif
  font->create();
  }


// Detach window
void FXGroupBox::detach(){
  FXPacker::detach();
  font->detach();
  }


// Enable the window
void FXGroupBox::enable(){
  if(!(flags&FLAG_ENABLED)){
    FXPacker::enable();
    update();
    }
  }


// Disable the window
void FXGroupBox::disable(){
  if(flags&FLAG_ENABLED){
    FXPacker::disable();
    update();
    }
  }


// Change the font
void FXGroupBox::setFont(FXFont* fnt){
  if(!fnt){ fxerror("%s::setFont: NULL font specified.\n",getClassName()); }
  if(font!=fnt){
    font=fnt;
    recalc();
    update();
    }
  }


// Get default width
FXint FXGroupBox::getDefaultWidth(){
  FXint cw=FXPacker::getDefaultWidth();
  if(!label.empty()){
    FXint tw=font->getTextWidth(label.text(),label.length())+16;
    return FXMAX(cw,tw);
    }
  return cw;
  }


// Get default height
FXint FXGroupBox::getDefaultHeight(){
  FXint ch=FXPacker::getDefaultHeight();
  if(!label.empty()){
    return ch+font->getFontHeight()+4;
    }
  return ch;
  }


// Recompute layout
void FXGroupBox::layout(){
  FXint tmp=padtop;
  if(!label.empty()){
    padtop=padtop+font->getFontHeight()+4-border;
    }
  FXPacker::layout();
  padtop=tmp;
  flags&=~FLAG_DIRTY;
  }


// Handle repaint
long FXGroupBox::onPaint(FXObject*,FXSelector,void* ptr){
  FXEvent *ev=(FXEvent*)ptr;
  FXDCWindow dc(this,ev);
  FXint tw,yy,xx,hh;

  tw=0;
  xx=0;
  yy=0;
  hh=height;

// CAE begin
  theme.reset(THEME_GROUPBOX);
  if(theme.active() && font==gbFont)
    font=getApp()->getNormalFont();
// CAE end

  // Draw label if there is one
  if(!label.empty()){
    tw=font->getTextWidth(label.text(),label.length());
    yy=2+font->getFontAscent()/2;
    hh=height-yy;
    }

  // CAE begin
  if (theme.active())
  {
      int stateId;
      if (isEnabled())
          stateId = GBS_NORMAL;
      else
          stateId = GBS_DISABLED;

      drawThemeBackground(dc,ev->rect.x,ev->rect.y,ev->rect.w,ev->rect.h);

      if (options&FRAME_MASK)
          theme.drawBackground( dc, BP_GROUPBOX, stateId, 0, yy, width, hh );

      // Clear the background behind the text
      if (!label.empty()) {
          textColor = theme.getThemeTextColor( BP_GROUPBOX, stateId );
                if(options&GROUPBOX_TITLE_RIGHT) xx=width-tw-16;
          else if(options&GROUPBOX_TITLE_CENTER) xx=(width-tw)/2-4;
          else xx=8;
          drawThemeBackground(dc,xx,0,tw+8,font->getFontHeight());
      }
  }
  else
  {
  font=gbFont;
  textColor=getApp()->getForeColor();
  // CAE end

  // Paint background
  dc.setForeground(backColor);
  dc.fillRectangle(ev->rect.x,ev->rect.y,ev->rect.w,ev->rect.h);

  // We should really just draw what's exposed!
  switch(options&FRAME_MASK) {
    case FRAME_LINE: drawBorderRectangle(dc,0,yy,width,hh); break;
    case FRAME_SUNKEN: drawSunkenRectangle(dc,0,yy,width,hh); break;
    case FRAME_RAISED: drawRaisedRectangle(dc,0,yy,width,hh); break;
    case FRAME_GROOVE: drawGrooveRectangle(dc,0,yy,width,hh); break;
    case FRAME_RIDGE: drawRidgeRectangle(dc,0,yy,width,hh); break;
    case FRAME_SUNKEN|FRAME_THICK: drawDoubleSunkenRectangle(dc,0,yy,width,hh); break;
    case FRAME_RAISED|FRAME_THICK: drawDoubleRaisedRectangle(dc,0,yy,width,hh); break;
    }

  } // CAE

  // Draw label
  if(!label.empty()){
    if(options&GROUPBOX_TITLE_RIGHT) xx=width-tw-16;
    else if(options&GROUPBOX_TITLE_CENTER) xx=(width-tw)/2-4;
    else xx=8;
    dc.setForeground(backColor);
    dc.setTextFont(font);
    if (!theme.active())             // CAE
    dc.fillRectangle(xx,yy,tw+8,2);
    if(isEnabled()){
      dc.setForeground(textColor);
      dc.drawText(xx+4,2+font->getFontAscent(),label.text(),label.length());
      }
    else{
      dc.setForeground(hiliteColor);
      dc.drawText(xx+5,3+font->getFontAscent(),label.text(),label.length());
      dc.setForeground(shadowColor);
      dc.drawText(xx+4,2+font->getFontAscent(),label.text(),label.length());
      }
    }
  return 1;
  }


// Uncheck other radio buttons
long FXGroupBox::onUncheckOther(FXObject* sender,FXSelector,void*){
  FXWindow* child;
  for(child=getFirst(); child; child=child->getNext()){
    if(child!=sender){ child->handle(sender,MKUINT(0,SEL_UNCHECK_RADIO),NULL); }
    }
  return 1;
  }


// Get group box style
FXuint FXGroupBox::getGroupBoxStyle() const {
  return (options&GROUPBOX_TITLE_MASK);
  }


// Set group box style
void FXGroupBox::setGroupBoxStyle(FXuint style){
  FXuint opts=(options&~GROUPBOX_TITLE_MASK) | (style&GROUPBOX_TITLE_MASK);
  if(options!=opts){
    options=opts;
    recalc();
    update();
    }
  }


// Change text
  // CAE use translated string for displayed text
void FXGroupBox::setText(const atr_String& text){
  if(label!=text){
    label=text;
    recalc();
    update();
    }
  }


// Set text color
void FXGroupBox::setTextColor(FXColor clr){
  if(textColor!=clr){
    textColor=clr;
    update();
    }
  }


// Save object to stream
void FXGroupBox::save(FXStream& store) const {
  FXPacker::save(store);
  store << label;
  store << font;
  store << textColor;
  }


// Load object from stream
void FXGroupBox::load(FXStream& store){
  FXPacker::load(store);
  store >> label;
  store >> font;
  store >> textColor;
  }
